@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -23,23 +24,21 @@
 #pragma once
 
 #include "DeflateBaseCompressor.h"
+#include "../Compatibility.h"  // For ORTHANC_OVERRIDE
 
 namespace Orthanc
 {
   class ORTHANC_PUBLIC ZlibCompressor : public DeflateBaseCompressor
   {
   public:
-    ZlibCompressor()
-    {
-      SetPrefixWithUncompressedSize(true);
-    }
+    ZlibCompressor();
 
     virtual void Compress(std::string& compressed,
                           const void* uncompressed,
-                          size_t uncompressedSize);
+                          size_t uncompressedSize) ORTHANC_OVERRIDE;
 
     virtual void Uncompress(std::string& uncompressed,
                             const void* compressed,
-                            size_t compressedSize);
+                            size_t compressedSize) ORTHANC_OVERRIDE;
   };
 }
