@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -138,6 +139,11 @@ namespace Orthanc
 
     void LogMissingTagsForStore() const;
 
+    static void LogMissingTagsForStore(const std::string& patientId,
+                                       const std::string& studyInstanceUid,
+                                       const std::string& seriesInstanceUid,
+                                       const std::string& sopInstanceUid);
+
     bool LookupStringValue(std::string& result,
                            const DicomTag& tag,
                            bool allowBinary) const;
