@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -26,6 +27,7 @@
 #include <set>
 #include <stdint.h>
 
+#include "../Compatibility.h"
 #include "../Enumerations.h"
 
 namespace Orthanc
@@ -40,50 +42,39 @@ namespace Orthanc
 
   public:
     DicomTag(uint16_t group,
-             uint16_t element) :
-      group_(group),
-      element_(element)
-    {
-    }
+             uint16_t element);
 
-    uint16_t GetGroup() const
-    {
-      return group_;
-    }
+    uint16_t GetGroup() const;
 
-    uint16_t GetElement() const
-    {
-      return element_;
-    }
+    uint16_t GetElement() const;
 
-    bool IsPrivate() const
-    {
-      return group_ % 2 == 1;
-    }
-
-    const char* GetMainTagsName() const;
+    bool IsPrivate() const;
 
     bool operator< (const DicomTag& other) const;
 
-    bool operator== (const DicomTag& other) const
-    {
-      return group_ == other.group_ && element_ == other.element_;
-    }
+    bool operator<= (const DicomTag& other) const;
 
-    bool operator!= (const DicomTag& other) const
-    {
-      return !(*this == other);
-    }
+    bool operator> (const DicomTag& other) const;
+
+    bool operator>= (const DicomTag& other) const;
+
+    bool operator== (const DicomTag& other) const;
+
+    bool operator!= (const DicomTag& other) const;
 
     std::string Format() const;
 
+    std::ostream& FormatStream(std::ostream& o) const;
+
     static bool ParseHexadecimal(DicomTag& tag,
                                  const char* value);
 
-    ORTHANC_PUBLIC friend std::ostream& operator<< (std::ostream& o, const DicomTag& tag);
-
     static void AddTagsForModule(std::set<DicomTag>& target,
                                  DicomModule module);
+
+#if ORTHANC_BUILDING_FRAMEWORK_LIBRARY == 1
+    ORTHANC_PUBLIC ORTHANC_DEPRECATED(friend std::ostream& operator<<(std::ostream& o, const DicomTag& tag));
+#endif
   };
 
   // Aliases for the most useful tags
@@ -182,6 +173,9 @@ namespace Orthanc
   static const DicomTag DICOM_TAG_PATIENT_SPECIES_DESCRIPTION(0x0010, 0x2201);
   static const DicomTag DICOM_TAG_STUDY_COMMENTS(0x0032, 0x4000);
   static const DicomTag DICOM_TAG_OTHER_PATIENT_IDS(0x0010, 0x1000);
+  static const DicomTag DICOM_TAG_PER_FRAME_FUNCTIONAL_GROUP_SEQUENCE(0x5200, 0x9230);
+  static const DicomTag DICOM_TAG_PIXEL_VALUE_TRANSFORMATION_SEQUENCE(0x0028, 0x9145);
+  static const DicomTag DICOM_TAG_FRAME_VOI_LUT_SEQUENCE(0x0028, 0x9132);
 
   // Tags used within the Stone of Orthanc
   static const DicomTag DICOM_TAG_FRAME_INCREMENT_POINTER(0x0028, 0x0009);
@@ -200,6 +194,7 @@ namespace Orthanc
   static const DicomTag DICOM_TAG_GREEN_PALETTE_COLOR_LOOKUP_TABLE_DESCRIPTOR(0x0028, 0x1102);
   static const DicomTag DICOM_TAG_BLUE_PALETTE_COLOR_LOOKUP_TABLE_DESCRIPTOR(0x0028, 0x1103);
   static const DicomTag DICOM_TAG_CONTOUR_DATA(0x3006, 0x0050);
+  static const DicomTag DICOM_TAG_CINE_RATE(0x0018, 0x0040);
                              
   // Counting patients, studies and series
   // https://www.medicalconnections.co.uk/kb/Counting_Studies_Series_and_Instances
