@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -57,6 +58,7 @@ namespace Orthanc
                         const std::string& remoteIp,
                         const std::string& remoteAet,
                         const std::string& calledAet,
+                        unsigned int maximumPduLength,
                         IApplicationEntityFilter* filter);
 
       virtual ~CommandDispatcher();
@@ -65,7 +67,9 @@ namespace Orthanc
     };
 
     CommandDispatcher* AcceptAssociation(const DicomServer& server, 
-                                         T_ASC_Network *net);
+                                         T_ASC_Network *net,
+                                         unsigned int maximumPduLength,
+                                         bool useDicomTls);
 
     OFCondition EchoScp(T_ASC_Association* assoc, 
                         T_DIMSE_Message* msg, 
