@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -32,9 +33,9 @@ namespace Orthanc
                        T_DIMSE_Message * msg, 
                        T_ASC_PresentationContextID presID,
                        IGetRequestHandler& handler,
-                       std::string remoteIp,
-                       std::string remoteAet,
-                       std::string calledAet,
+                       const std::string& remoteIp,
+                       const std::string& remoteAet,
+                       const std::string& calledAet,
                        int timeout);
   }
 }
