@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -40,6 +41,7 @@ namespace Orthanc
     bool                  allowEcho_;
     bool                  allowStore_;
     bool                  allowFind_;
+    bool                  allowFindWorklist_;
     bool                  allowMove_;
     bool                  allowGet_;
     bool                  allowNAction_;
@@ -47,6 +49,7 @@ namespace Orthanc
     bool                  allowTranscoding_;
     bool                  useDicomTls_;
     std::string           localAet_;
+    uint32_t              timeout_;
     
     void Clear();
 
@@ -107,5 +110,12 @@ namespace Orthanc
     const std::string& GetLocalAet() const;
 
     void SetLocalAet(const std::string& aet);
+
+    // Setting it to "0" will use "DicomAssociationParameters::GetDefaultTimeout()"
+    void SetTimeout(uint32_t seconds);
+
+    uint32_t GetTimeout() const;
+
+    bool HasTimeout() const;    
   };
 }
