@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -41,11 +42,9 @@ namespace Orthanc
     // Debug mode
     ParsedDicomFile parsed(transcoded);
 
-    std::string s;
     DicomTransferSyntax a, b;
-    if (!const_cast<const ParsedDicomFile&>(parsed).LookupTransferSyntax(s) ||
+    if (!const_cast<const ParsedDicomFile&>(parsed).LookupTransferSyntax(b) ||
         !FromDcmtkBridge::LookupOrthancTransferSyntax(a, parsed.GetDcmtkObject()) ||
-        !LookupTransferSyntax(b, s) ||
         a != b ||
         allowedSyntaxes.find(a) == allowedSyntaxes.end())
     {
