@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,8 +26,10 @@
 #include "../IMemoryBuffer.h"
 #include "../Enumerations.h"
 
+#include <stdint.h>
 #include <string>
 
+
 namespace Orthanc
 {
   class IStorageArea : public boost::noncopyable
@@ -44,6 +47,13 @@ namespace Orthanc
     virtual IMemoryBuffer* Read(const std::string& uuid,
                                 FileContentType type) = 0;
 
+    virtual IMemoryBuffer* ReadRange(const std::string& uuid,
+                                     FileContentType type,
+                                     uint64_t start /* inclusive */,
+                                     uint64_t end /* exclusive */) = 0;
+
+    virtual bool HasReadRange() const = 0;
+
     virtual void Remove(const std::string& uuid,
                         FileContentType type) = 0;
   };
