@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -91,6 +92,61 @@ namespace Orthanc
   }
       
 
+  IMemoryBuffer* MemoryStorageArea::ReadRange(const std::string& uuid,
+                                              FileContentType type,
+                                              uint64_t start /* inclusive */,
+                                              uint64_t end /* exclusive */)
+  {
+    LOG(INFO) << "Reading attachment \"" << uuid << "\" of \""
+              << static_cast<int>(type) << "\" content type "
+              << "(range from " << start << " to " << end << ")";
+
+    if (start > end)
+    {
+      throw OrthancException(ErrorCode_BadRange);
+    }
+    else if (start == end)
+    {
+      return new StringMemoryBuffer;
+    }
+    else
+    {
+      boost::mutex::scoped_lock lock(mutex_);
+
+      Content::const_iterator found = content_.find(uuid);
+
+      if (found == content_.end())
+      {
+        throw OrthancException(ErrorCode_InexistentFile);
+      }
+      else if (found->second == NULL)
+      {
+        throw OrthancException(ErrorCode_InternalError);
+      }
+      else if (end > found->second->size())
+      {
+        throw OrthancException(ErrorCode_BadRange);
+      }
+      else
+      {
+        std::string range;
+        range.resize(end - start);
+        assert(!range.empty());
+
+        memcpy(&range[0], &found->second[start], range.size());
+        
+        return StringMemoryBuffer::CreateFromSwap(range);
+      }
+    }
+  }
+
+
+  bool MemoryStorageArea::HasReadRange() const
+  {
+    return true;
+  }
+
+
   void MemoryStorageArea::Remove(const std::string& uuid,
                                  FileContentType type)
   {
