@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -54,6 +55,7 @@
 namespace Orthanc
 {
   class MetricsRegistry;
+  class StorageCache;
 
   /**
    * This class handles the compression/decompression of the raw files
@@ -66,6 +68,7 @@ namespace Orthanc
     class MetricsTimer;
 
     IStorageArea&     area_;
+    StorageCache&     cache_;
     MetricsRegistry*  metrics_;
 
 #if ORTHANC_ENABLE_CIVETWEB == 1 || ORTHANC_ENABLE_MONGOOSE == 1
@@ -75,18 +78,12 @@ namespace Orthanc
 #endif
 
   public:
-    StorageAccessor(IStorageArea& area) : 
-      area_(area),
-      metrics_(NULL)
-    {
-    }
+    explicit StorageAccessor(IStorageArea& area,
+                             StorageCache& cache);
 
     StorageAccessor(IStorageArea& area,
-                    MetricsRegistry& metrics) : 
-      area_(area),
-      metrics_(&metrics)
-    {
-    }
+                    StorageCache& cache,
+                    MetricsRegistry& metrics);
 
     FileInfo Write(const void* data,
                    size_t size,
@@ -94,14 +91,10 @@ namespace Orthanc
                    CompressionType compression,
                    bool storeMd5);
 
-    FileInfo Write(const std::string& data, 
+    FileInfo Write(const std::string& data,
                    FileContentType type,
                    CompressionType compression,
-                   bool storeMd5)
-    {
-      return Write((data.size() == 0 ? NULL : data.c_str()),
-                   data.size(), type, compression, storeMd5);
-    }
+                   bool storeMd5);
 
     void Read(std::string& content,
               const FileInfo& info);
@@ -109,21 +102,20 @@ namespace Orthanc
     void ReadRaw(std::string& content,
                  const FileInfo& info);
 
+    void ReadStartRange(std::string& target,
+                        const std::string& fileUuid,
+                        FileContentType fullFileContentType,
+                        uint64_t end /* exclusive */);
+
     void Remove(const std::string& fileUuid,
                 FileContentType type);
 
-    void Remove(const FileInfo& info)
-    {
-      Remove(info.GetUuid(), info.GetContentType());
-    }
+    void Remove(const FileInfo& info);
 
 #if ORTHANC_ENABLE_CIVETWEB == 1 || ORTHANC_ENABLE_MONGOOSE == 1
     void AnswerFile(HttpOutput& output,
                     const FileInfo& info,
-                    MimeType mime)
-    {
-      AnswerFile(output, info, EnumerationToString(mime));
-    }
+                    MimeType mime);
 
     void AnswerFile(HttpOutput& output,
                     const FileInfo& info,
@@ -131,10 +123,7 @@ namespace Orthanc
 
     void AnswerFile(RestApiOutput& output,
                     const FileInfo& info,
-                    MimeType mime)
-    {
-      AnswerFile(output, info, EnumerationToString(mime));
-    }
+                    MimeType mime);
 
     void AnswerFile(RestApiOutput& output,
                     const FileInfo& info,
