@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -65,7 +66,7 @@ extern "C"
 // This is a dummy wrapper function to suppress any OpenSSL-related
 // problem in valgrind. Inlining is prevented.
 #if defined(__GNUC__) || defined(__clang__)
-  __attribute__((noinline)) 
+__attribute__((noinline)) 
 #endif
 static CURLcode OrthancHttpClientPerformSSL(CURL* curl, long* status)
 {
@@ -479,7 +480,7 @@ namespace Orthanc
 
     void SetDefaultProxy(const std::string& proxy)
     {
-      LOG(INFO) << "Setting the default proxy for HTTP client connections: " << proxy;
+      CLOG(INFO, HTTP) << "Setting the default proxy for HTTP client connections: " << proxy;
 
       {
         boost::mutex::scoped_lock lock(mutex_);
@@ -495,7 +496,7 @@ namespace Orthanc
 
     void SetDefaultTimeout(long seconds)
     {
-      LOG(INFO) << "Setting the default timeout for HTTP client connections: " << seconds << " seconds";
+      CLOG(INFO, HTTP) << "Setting the default timeout for HTTP client connections: " << seconds << " seconds";
 
       {
         boost::mutex::scoped_lock lock(mutex_);
@@ -568,29 +569,29 @@ namespace Orthanc
 
 
   /*static int CurlDebugCallback(CURL *handle,
-                               curl_infotype type,
-                               char *data,
-                               size_t size,
-                               void *userptr)
-  {
+    curl_infotype type,
+    char *data,
+    size_t size,
+    void *userptr)
+    {
     switch (type)
     {
-      case CURLINFO_TEXT:
-      case CURLINFO_HEADER_IN:
-      case CURLINFO_HEADER_OUT:
-      case CURLINFO_SSL_DATA_IN:
-      case CURLINFO_SSL_DATA_OUT:
-      case CURLINFO_END:
-      case CURLINFO_DATA_IN:
-      case CURLINFO_DATA_OUT:
-      {
-        std::string s(data, size);
-        LOG(INFO) << "libcurl: " << s;
-        break;
-      }
+    case CURLINFO_TEXT:
+    case CURLINFO_HEADER_IN:
+    case CURLINFO_HEADER_OUT:
+    case CURLINFO_SSL_DATA_IN:
+    case CURLINFO_SSL_DATA_OUT:
+    case CURLINFO_END:
+    case CURLINFO_DATA_IN:
+    case CURLINFO_DATA_OUT:
+    {
+    std::string s(data, size);
+    CLOG(INFO, INFO) << "libcurl: " << s;
+    break;
+    }
 
-      default:
-        break;
+    default:
+    break;
     }
 
     return 0;
@@ -622,6 +623,10 @@ namespace Orthanc
     timeout_ = GlobalParameters::GetInstance().GetDefaultTimeout();
     GlobalParameters::GetInstance().GetDefaultProxy(proxy_);
     GlobalParameters::GetInstance().GetSslConfiguration(verifyPeers_, caCertificates_);    
+
+    hasExternalBody_ = false;
+    externalBodyData_ = NULL;
+    externalBodySize_ = 0;
   }
 
 
@@ -669,6 +674,11 @@ namespace Orthanc
     {
       AddHeader(it->first, it->second);
     }
+
+    if (service.HasTimeout())
+    {
+      SetTimeout(service.GetTimeout());
+    }
   }
 
 
@@ -713,20 +723,28 @@ namespace Orthanc
   }
 
 
-  void HttpClient::SetBody(const std::string& data)
+  void HttpClient::AssignBody(const std::string& data)
   {
     body_ = data;
     pimpl_->requestBody_.Clear();
+    hasExternalBody_ = false;
   }
 
-  std::string &HttpClient::GetBody()
-  {
-    return body_;
-  }
 
-  const std::string &HttpClient::GetBody() const
+  void HttpClient::AssignBody(const void* data,
+                              size_t size)
   {
-    return body_;
+    if (size != 0 &&
+        data == NULL)
+    {
+      throw OrthancException(ErrorCode_NullPointer);
+    }
+    else
+    {
+      body_.assign(reinterpret_cast<const char*>(data), size);
+      pimpl_->requestBody_.Clear();
+      hasExternalBody_ = false;
+    }
   }
 
 
@@ -734,13 +752,40 @@ namespace Orthanc
   {
     body_.clear();
     pimpl_->requestBody_.SetBody(body);
+    hasExternalBody_ = false;
   }
 
   
+  void HttpClient::SetExternalBody(const void* data,
+                                   size_t size)
+  {
+    if (size != 0 &&
+        data == NULL)
+    {
+      throw OrthancException(ErrorCode_NullPointer);
+    }
+    else
+    {
+      body_.clear();
+      pimpl_->requestBody_.Clear();
+      hasExternalBody_ = true;
+      externalBodyData_ = data;
+      externalBodySize_ = size;
+    }
+  }
+  
+
+  void HttpClient::SetExternalBody(const std::string& data)
+  {
+    SetExternalBody(data.empty() ? NULL : data.c_str(), data.size());
+  }
+
+
   void HttpClient::ClearBody()
   {
     body_.clear();
     pimpl_->requestBody_.Clear();
+    hasExternalBody_ = false;
   }
 
 
@@ -787,6 +832,9 @@ namespace Orthanc
 
   bool HttpClient::ApplyInternal(CurlAnswer& answer)
   {
+    CLOG(INFO, HTTP) << "New HTTP request to: " << url_ << " (timeout: "
+                     << boost::lexical_cast<std::string>(timeout_ <= 0 ? DEFAULT_HTTP_TIMEOUT : timeout_) << "s)";
+    
     CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_URL, url_.c_str()));
     CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_HEADERDATA, &answer));
 
@@ -898,31 +946,31 @@ namespace Orthanc
 
     switch (method_)
     {
-    case HttpMethod_Get:
-      CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_HTTPGET, 1L));
-      break;
+      case HttpMethod_Get:
+        CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_HTTPGET, 1L));
+        break;
 
-    case HttpMethod_Post:
-      CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_POST, 1L));
+      case HttpMethod_Post:
+        CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_POST, 1L));
 
-      break;
+        break;
 
-    case HttpMethod_Delete:
-      CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_NOBODY, 1L));
-      CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_CUSTOMREQUEST, "DELETE"));
-      break;
+      case HttpMethod_Delete:
+        CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_NOBODY, 1L));
+        CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_CUSTOMREQUEST, "DELETE"));
+        break;
 
-    case HttpMethod_Put:
-      // http://stackoverflow.com/a/7570281/881731: Don't use
-      // CURLOPT_PUT if there is a body
+      case HttpMethod_Put:
+        // http://stackoverflow.com/a/7570281/881731: Don't use
+        // CURLOPT_PUT if there is a body
 
-      // CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_PUT, 1L));
+        // CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_PUT, 1L));
 
-      curl_easy_setopt(pimpl_->curl_, CURLOPT_CUSTOMREQUEST, "PUT"); /* !!! */
-      break;
+        curl_easy_setopt(pimpl_->curl_, CURLOPT_CUSTOMREQUEST, "PUT"); /* !!! */
+        break;
 
-    default:
-      throw OrthancException(ErrorCode_InternalError);
+      default:
+        throw OrthancException(ErrorCode_InternalError);
     }
 
     if (method_ == HttpMethod_Post ||
@@ -931,7 +979,7 @@ namespace Orthanc
       if (!pimpl_->userHeaders_.IsEmpty() &&
           !pimpl_->userHeaders_.HasExpect())
       {
-        LOG(INFO) << "For performance, the HTTP header \"Expect\" should be set to empty string in POST/PUT requests";
+        CLOG(INFO, HTTP) << "For performance, the HTTP header \"Expect\" should be set to empty string in POST/PUT requests";
       }
 
       if (pimpl_->requestBody_.IsValid())
@@ -968,7 +1016,12 @@ namespace Orthanc
           pimpl_->defaultPostHeaders_.Assign(pimpl_->curl_);
         }
 
-        if (body_.size() > 0)
+        if (hasExternalBody_)
+        {
+          CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_POSTFIELDS, externalBodyData_));
+          CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_POSTFIELDSIZE, externalBodySize_));
+        }
+        else if (body_.size() > 0)
         {
           CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_POSTFIELDS, body_.c_str()));
           CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_POSTFIELDSIZE, body_.size()));
@@ -1001,13 +1054,13 @@ namespace Orthanc
 
     const boost::posix_time::ptime end = boost::posix_time::microsec_clock::universal_time();
     
-    LOG(INFO) << "HTTP status code " << status << " in "
-              << ((end - start).total_milliseconds()) << " ms after "
-              << EnumerationToString(method_) << " request on: " << url_;
+    CLOG(INFO, HTTP) << "HTTP status code " << status << " in "
+                     << ((end - start).total_milliseconds()) << " ms after "
+                     << EnumerationToString(method_) << " request on: " << url_;
 
     if (isVerbose_)
     {
-      LOG(INFO) << "cURL status code: " << code;
+      CLOG(INFO, HTTP) << "cURL status code: " << code;
     }
 
     CheckCode(code);
@@ -1333,8 +1386,8 @@ namespace Orthanc
                                     bool verbose)
   {
 #if ORTHANC_ENABLE_PKCS11 == 1
-    LOG(INFO) << "Initializing PKCS#11 using " << module 
-              << (pin.empty() ? " (no PIN provided)" : " (PIN is provided)");
+    CLOG(INFO, HTTP) << "Initializing PKCS#11 using " << module 
+                     << (pin.empty() ? " (no PIN provided)" : " (PIN is provided)");
     GlobalParameters::GetInstance().InitializePkcs11(module, pin, verbose);    
 #else
     throw OrthancException(ErrorCode_InternalError,
