@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -29,7 +30,7 @@
 #include <string>
 #include <boost/noncopyable.hpp>
 #include <boost/shared_ptr.hpp>
-#include <json/json.h>
+#include <json/value.h>
 
 #if !defined(ORTHANC_ENABLE_CURL)
 #  error The macro ORTHANC_ENABLE_CURL must be defined
@@ -106,6 +107,11 @@ namespace Orthanc
     bool headersToLowerCase_;
     bool redirectionFollowed_;
 
+    // New in Orthanc 1.9.3 to avoid memcpy()
+    bool        hasExternalBody_;
+    const void* externalBodyData_;
+    size_t      externalBodySize_;
+
     void Setup();
 
     void operator= (const HttpClient&);  // Assignment forbidden
@@ -127,63 +133,39 @@ namespace Orthanc
 
     ~HttpClient();
 
-    void SetUrl(const char* url)
-    {
-      url_ = std::string(url);
-    }
+    void SetUrl(const char* url);
 
-    void SetUrl(const std::string& url)
-    {
-      url_ = url;
-    }
+    void SetUrl(const std::string& url);
 
-    const std::string& GetUrl() const
-    {
-      return url_;
-    }
+    const std::string& GetUrl() const;
 
-    void SetMethod(HttpMethod method)
-    {
-      method_ = method;
-    }
+    void SetMethod(HttpMethod method);
 
-    HttpMethod GetMethod() const
-    {
-      return method_;
-    }
+    HttpMethod GetMethod() const;
 
-    void SetTimeout(long seconds)
-    {
-      timeout_ = seconds;
-    }
+    void SetTimeout(long seconds);
 
-    long GetTimeout() const
-    {
-      return timeout_;
-    }
+    long GetTimeout() const;
 
-    void SetBody(const std::string& data);
+    void AssignBody(const std::string& data);
 
-    std::string& GetBody()
-    {
-      return body_;
-    }
-
-    const std::string& GetBody() const
-    {
-      return body_;
-    }
+    void AssignBody(const void* data,
+                    size_t size);
 
     void SetBody(IRequestBody& body);
 
+    // New in Orthanc 1.9.3: The "data" buffer must have a lifetime
+    // that is longer than the HttpClient object
+    void SetExternalBody(const void* data,
+                         size_t size);
+
+    void SetExternalBody(const std::string& data);
+
     void ClearBody();
 
     void SetVerbose(bool isVerbose);
 
-    bool IsVerbose() const
-    {
-      return isVerbose_;
-    }
+    bool IsVerbose() const;
 
     void AddHeader(const std::string& key,
                    const std::string& value);
@@ -192,109 +174,52 @@ namespace Orthanc
 
     bool Apply(IAnswer& answer);
 
-    bool Apply(std::string& answerBody)
-    {
-      return ApplyInternal(answerBody, NULL);
-    }
+    bool Apply(std::string& answerBody);
 
-    bool Apply(Json::Value& answerBody)
-    {
-      return ApplyInternal(answerBody, NULL);
-    }
+    bool Apply(Json::Value& answerBody);
 
     bool Apply(std::string& answerBody,
-               HttpHeaders& answerHeaders)
-    {
-      return ApplyInternal(answerBody, &answerHeaders);
-    }
+               HttpHeaders& answerHeaders);
 
     bool Apply(Json::Value& answerBody,
-               HttpHeaders& answerHeaders)
-    {
-      return ApplyInternal(answerBody, &answerHeaders);
-    }
+               HttpHeaders& answerHeaders);
 
-    HttpStatus GetLastStatus() const
-    {
-      return lastStatus_;
-    }
+    HttpStatus GetLastStatus() const;
 
     void SetCredentials(const char* username,
                         const char* password);
 
-    void SetProxy(const std::string& proxy)
-    {
-      proxy_ = proxy;
-    }
+    void SetProxy(const std::string& proxy);
 
-    void SetHttpsVerifyPeers(bool verify)
-    {
-      verifyPeers_ = verify;
-    }
+    void SetHttpsVerifyPeers(bool verify);
 
-    bool IsHttpsVerifyPeers() const
-    {
-      return verifyPeers_;
-    }
+    bool IsHttpsVerifyPeers() const;
 
-    void SetHttpsCACertificates(const std::string& certificates)
-    {
-      caCertificates_ = certificates;
-    }
+    void SetHttpsCACertificates(const std::string& certificates);
 
-    const std::string& GetHttpsCACertificates() const
-    {
-      return caCertificates_;
-    }
+    const std::string& GetHttpsCACertificates() const;
 
     void SetClientCertificate(const std::string& certificateFile,
                               const std::string& certificateKeyFile,
                               const std::string& certificateKeyPassword);
 
-    void SetPkcs11Enabled(bool enabled)
-    {
-      pkcs11Enabled_ = enabled;
-    }
+    void SetPkcs11Enabled(bool enabled);
 
-    bool IsPkcs11Enabled() const
-    {
-      return pkcs11Enabled_;
-    }
+    bool IsPkcs11Enabled() const;
 
-    const std::string& GetClientCertificateFile() const
-    {
-      return clientCertificateFile_;
-    }
+    const std::string& GetClientCertificateFile() const;
 
-    const std::string& GetClientCertificateKeyFile() const
-    {
-      return clientCertificateKeyFile_;
-    }
+    const std::string& GetClientCertificateKeyFile() const;
 
-    const std::string& GetClientCertificateKeyPassword() const
-    {
-      return clientCertificateKeyPassword_;
-    }
+    const std::string& GetClientCertificateKeyPassword() const;
 
-    void SetConvertHeadersToLowerCase(bool lowerCase)
-    {
-      headersToLowerCase_ = lowerCase;
-    }
+    void SetConvertHeadersToLowerCase(bool lowerCase);
 
-    bool IsConvertHeadersToLowerCase() const
-    {
-      return headersToLowerCase_;
-    }
+    bool IsConvertHeadersToLowerCase() const;
 
-    void SetRedirectionFollowed(bool follow)
-    {
-      redirectionFollowed_ = follow;
-    }
+    void SetRedirectionFollowed(bool follow);
 
-    bool IsRedirectionFollowed() const
-    {
-      return redirectionFollowed_;
-    }
+    bool IsRedirectionFollowed() const;
 
     static void GlobalInitialize();
   
