@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -41,7 +42,7 @@ namespace Orthanc
 
 
   static void OutputDirectoryContent(HttpOutput& output,
-                                     const IHttpHandler::Arguments& headers,
+                                     const HttpToolbox::Arguments& headers,
                                      const UriComponents& uri,
                                      const boost::filesystem::path& p)
   {
@@ -117,17 +118,16 @@ namespace Orthanc
   }
 
 
-  bool FilesystemHttpHandler::Handle(
-    HttpOutput& output,
-    RequestOrigin /*origin*/,
-    const char* /*remoteIp*/,
-    const char* /*username*/,
-    HttpMethod method,
-    const UriComponents& uri,
-    const Arguments& headers,
-    const GetArguments& arguments,
-    const void* /*bodyData*/,
-    size_t /*bodySize*/)
+  bool FilesystemHttpHandler::Handle(HttpOutput& output,
+                                     RequestOrigin /*origin*/,
+                                     const char* /*remoteIp*/,
+                                     const char* /*username*/,
+                                     HttpMethod method,
+                                     const UriComponents& uri,
+                                     const HttpToolbox::Arguments& headers,
+                                     const HttpToolbox::GetArguments& arguments,
+                                     const void* /*bodyData*/,
+                                     size_t /*bodySize*/)
   {
     if (!Toolbox::IsChildUri(pimpl_->baseUri_, uri))
     {
