@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -51,6 +52,7 @@ namespace Orthanc
     state_(State_WritingHeader),
     status_(HttpStatus_200_Ok),
     hasContentLength_(false),
+    contentLength_(0),
     contentPosition_(0),
     keepAlive_(isKeepAlive)
   {
@@ -173,6 +175,21 @@ namespace Orthanc
       if (keepAlive_)
       {
         s += "Connection: keep-alive\r\n";
+
+        /**
+         * [LIFY-2311] The "Keep-Alive" HTTP header was missing in
+         * Orthanc <= 1.8.0, which notably caused failures if
+         * uploading DICOM instances by applying Java's
+         * "org.apache.http.client.methods.HttpPost()" on "/instances"
+         * URI, if "PoolingHttpClientConnectionManager" was in used. A
+         * workaround was to manually set a timeout for the keep-alive
+         * client to, say, 200 milliseconds, by using
+         * "HttpClients.custom().setKeepAliveStrategy((httpResponse,httpContext)->200)".
+         * Note that the "timeout" value can only be integer in the
+         * HTTP header, so we can't use the milliseconds granularity.
+         **/
+        s += ("Keep-Alive: timeout=" +
+              boost::lexical_cast<std::string>(CIVETWEB_KEEP_ALIVE_TIMEOUT_SECONDS) + "\r\n");
       }
       else
       {
@@ -281,6 +298,35 @@ namespace Orthanc
   }
 
 
+  HttpOutput::HttpOutput(IHttpOutputStream &stream,
+                         bool isKeepAlive) :
+    stateMachine_(stream, isKeepAlive),
+    isDeflateAllowed_(false),
+    isGzipAllowed_(false)
+  {
+  }
+
+  void HttpOutput::SetDeflateAllowed(bool allowed)
+  {
+    isDeflateAllowed_ = allowed;
+  }
+
+  bool HttpOutput::IsDeflateAllowed() const
+  {
+    return isDeflateAllowed_;
+  }
+
+  void HttpOutput::SetGzipAllowed(bool allowed)
+  {
+    isGzipAllowed_ = allowed;
+  }
+
+  bool HttpOutput::IsGzipAllowed() const
+  {
+    return isGzipAllowed_;
+  }
+
+
   void HttpOutput::SendMethodNotAllowed(const std::string& allowed)
   {
     stateMachine_.ClearHeaders();
@@ -306,6 +352,41 @@ namespace Orthanc
     stateMachine_.SendBody(message, messageSize);
   }
 
+  void HttpOutput::SendStatus(HttpStatus status)
+  {
+    SendStatus(status, NULL, 0);
+  }
+
+  void HttpOutput::SendStatus(HttpStatus status, const std::string &message)
+  {
+    SendStatus(status, message.c_str(), message.size());
+  }
+
+  void HttpOutput::SetContentType(MimeType contentType)
+  {
+    stateMachine_.SetContentType(EnumerationToString(contentType));
+  }
+
+  void HttpOutput::SetContentType(const std::string &contentType)
+  {
+    stateMachine_.SetContentType(contentType.c_str());
+  }
+
+  void HttpOutput::SetContentFilename(const char *filename)
+  {
+    stateMachine_.SetContentFilename(filename);
+  }
+
+  void HttpOutput::SetCookie(const std::string &cookie, const std::string &value)
+  {
+    stateMachine_.SetCookie(cookie, value);
+  }
+
+  void HttpOutput::AddHeader(const std::string &key, const std::string &value)
+  {
+    stateMachine_.AddHeader(key, value);
+  }
+
 
   void HttpOutput::Redirect(const std::string& path)
   {
@@ -324,8 +405,30 @@ namespace Orthanc
     stateMachine_.SendBody(NULL, 0);
   }
 
+  void HttpOutput::StartMultipart(const std::string &subType, const std::string &contentType)
+  {
+    stateMachine_.StartMultipart(subType, contentType);
+  }
+
+  void HttpOutput::SendMultipartItem(const void *item,
+                                     size_t size,
+                                     const std::map<std::string, std::string> &headers)
+  {
+    stateMachine_.SendMultipartItem(item, size, headers);
+  }
+
+  void HttpOutput::CloseMultipart()
+  {
+    stateMachine_.CloseMultipart();
+  }
+
+  bool HttpOutput::IsWritingMultipart() const
+  {
+    return stateMachine_.GetState() == StateMachine::State_WritingMultipart;
+  }
+
   
-  void HttpOutput::Answer(const void* buffer, 
+  void HttpOutput::Answer(const void* buffer,
                           size_t length)
   {
     if (length == 0)
@@ -436,7 +539,7 @@ namespace Orthanc
      * Full history is available at the following locations:
      * - In changeset 2248:69b0f4e8a49b:
      *   # hg history -v -r 2248
-     * - https://bitbucket.org/sjodogne/orthanc/issues/54/
+     * - https://bugs.orthanc-server.com/show_bug.cgi?id=54
      * - https://groups.google.com/d/msg/orthanc-users/65zhIM5xbKI/TU5Q1_LhAwAJ
      **/
     std::string tmp;
@@ -458,7 +561,7 @@ namespace Orthanc
      * within the encapsulations, and must be no longer than 70
      * characters, not counting the two leading hyphens."
      * https://tools.ietf.org/html/rfc1521
-     * https://bitbucket.org/sjodogne/orthanc/issues/165/
+     * https://bugs.orthanc-server.com/show_bug.cgi?id=165
      **/
     if (boundary.size() != 36 + 1 + 36)  // one UUID contains 36 characters
     {
@@ -471,8 +574,7 @@ namespace Orthanc
   }
 
 
-  void HttpOutput::StateMachine::StartMultipart(const std::string& subType,
-                                                const std::string& contentType)
+  void HttpOutput::StateMachine::StartStreamInternal(const std::string& contentType)
   {
     if (state_ != State_WritingHeader)
     {
@@ -518,25 +620,40 @@ namespace Orthanc
       header += "Connection: close\r\n";
     }
 
-    // Possibly add the cookies
-    CheckHeadersCompatibilityWithMultipart();
-
     for (std::list<std::string>::const_iterator
            it = headers_.begin(); it != headers_.end(); ++it)
     {
       header += *it;
     }
 
+    header += ("Content-Type: " + contentType + "\r\n\r\n");
+
+    stream_.Send(true, header.c_str(), header.size());
+  }
+
+
+  void HttpOutput::StateMachine::StartMultipart(const std::string& subType,
+                                                const std::string& contentType)
+  {
+    CheckHeadersCompatibilityWithMultipart();
+
     std::string contentTypeHeader;
     PrepareMultipartMainHeader(multipartBoundary_, contentTypeHeader, subType, contentType);
     multipartContentType_ = contentType;
-    header += ("Content-Type: " + contentTypeHeader + "\r\n\r\n");
 
-    stream_.Send(true, header.c_str(), header.size());
+    StartStreamInternal(contentTypeHeader);
+
     state_ = State_WritingMultipart;
   }
 
 
+  void HttpOutput::StateMachine::StartStream(const std::string& contentType)
+  {
+    StartStreamInternal(contentType);
+    state_ = State_WritingStream;
+  }
+
+
   static void PrepareMultipartItemHeader(std::string& target,
                                          size_t length,
                                          const std::map<std::string, std::string>& headers,
@@ -634,6 +751,36 @@ namespace Orthanc
   }
 
 
+  void HttpOutput::StateMachine::SendStreamItem(const void* data,
+                                                size_t size)
+  {
+    if (state_ != State_WritingStream)
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else
+    {
+      if (size > 0)
+      {
+        stream_.Send(false, data, size);
+      }
+    }
+  }
+  
+
+  void HttpOutput::StateMachine::CloseStream()
+  {
+    if (state_ != State_WritingStream)
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else
+    {
+      state_ = State_Done;
+    }
+  }
+
+
   static void AnswerStreamAsBuffer(HttpOutput& output,
                                    IHttpStreamAnswer& stream)
   {
@@ -758,4 +905,29 @@ namespace Orthanc
     chunked.Flatten(body);
     Answer(body);
   }
+
+
+  void HttpOutput::AnswerWithoutBuffering(IHttpStreamAnswer& stream)
+  {
+    std::string contentType = stream.GetContentType();
+    if (contentType.empty())
+    {
+      contentType = MIME_BINARY;
+    }
+
+    std::string filename;
+    if (stream.HasContentFilename(filename))
+    {
+      stateMachine_.AddHeader("Content-Disposition", "filename=\"" + std::string(filename) + "\"");
+    }
+
+    stateMachine_.StartStream(contentType.c_str());
+
+    while (stream.ReadNextChunk())
+    {
+      stateMachine_.SendStreamItem(stream.GetChunkContent(), stream.GetChunkSize());
+    }
+
+    stateMachine_.CloseStream();
+  }
 }
