@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -47,7 +48,8 @@ namespace Orthanc
         State_WritingHeader,      
         State_WritingBody,
         State_WritingMultipart,
-        State_Done
+        State_Done,
+        State_WritingStream
       };
 
     private:
@@ -64,6 +66,8 @@ namespace Orthanc
       std::string multipartBoundary_;
       std::string multipartContentType_;
 
+      void StartStreamInternal(const std::string& contentType);
+
     public:
       StateMachine(IHttpOutputStream& stream,
                    bool isKeepAlive);
@@ -105,6 +109,13 @@ namespace Orthanc
       }
 
       void CheckHeadersCompatibilityWithMultipart() const;
+
+      void StartStream(const std::string& contentType);
+
+      void SendStreamItem(const void* data,
+                          size_t size);
+
+      void CloseStream();
     };
 
     StateMachine stateMachine_;
@@ -115,74 +126,36 @@ namespace Orthanc
 
   public:
     HttpOutput(IHttpOutputStream& stream,
-               bool isKeepAlive) : 
-      stateMachine_(stream, isKeepAlive),
-      isDeflateAllowed_(false),
-      isGzipAllowed_(false)
-    {
-    }
+               bool isKeepAlive);
 
-    void SetDeflateAllowed(bool allowed)
-    {
-      isDeflateAllowed_ = allowed;
-    }
+    void SetDeflateAllowed(bool allowed);
 
-    bool IsDeflateAllowed() const
-    {
-      return isDeflateAllowed_;
-    }
+    bool IsDeflateAllowed() const;
 
-    void SetGzipAllowed(bool allowed)
-    {
-      isGzipAllowed_ = allowed;
-    }
+    void SetGzipAllowed(bool allowed);
 
-    bool IsGzipAllowed() const
-    {
-      return isGzipAllowed_;
-    }
+    bool IsGzipAllowed() const;
 
     void SendStatus(HttpStatus status,
 		    const char* message,
 		    size_t messageSize);
 
-    void SendStatus(HttpStatus status)
-    {
-      SendStatus(status, NULL, 0);
-    }
+    void SendStatus(HttpStatus status);
 
     void SendStatus(HttpStatus status,
-		    const std::string& message)
-    {
-      SendStatus(status, message.c_str(), message.size());
-    }
+                    const std::string& message);
 
-    void SetContentType(MimeType contentType)
-    {
-      stateMachine_.SetContentType(EnumerationToString(contentType));
-    }
+    void SetContentType(MimeType contentType);
     
-    void SetContentType(const std::string& contentType)
-    {
-      stateMachine_.SetContentType(contentType.c_str());
-    }
+    void SetContentType(const std::string& contentType);
 
-    void SetContentFilename(const char* filename)
-    {
-      stateMachine_.SetContentFilename(filename);
-    }
+    void SetContentFilename(const char* filename);
 
     void SetCookie(const std::string& cookie,
-                   const std::string& value)
-    {
-      stateMachine_.SetCookie(cookie, value);
-    }
+                   const std::string& value);
 
     void AddHeader(const std::string& key,
-                   const std::string& value)
-    {
-      stateMachine_.AddHeader(key, value);
-    }
+                   const std::string& value);
 
     void Answer(const void* buffer, 
                 size_t length);
@@ -198,27 +171,15 @@ namespace Orthanc
     void SendUnauthorized(const std::string& realm);
 
     void StartMultipart(const std::string& subType,
-                        const std::string& contentType)
-    {
-      stateMachine_.StartMultipart(subType, contentType);
-    }
+                        const std::string& contentType);
 
-    void SendMultipartItem(const void* item, 
+    void SendMultipartItem(const void* item,
                            size_t size,
-                           const std::map<std::string, std::string>& headers)
-    {
-      stateMachine_.SendMultipartItem(item, size, headers);
-    }
+                           const std::map<std::string, std::string>& headers);
 
-    void CloseMultipart()
-    {
-      stateMachine_.CloseMultipart();
-    }
+    void CloseMultipart();
 
-    bool IsWritingMultipart() const
-    {
-      return stateMachine_.GetState() == StateMachine::State_WritingMultipart;
-    }
+    bool IsWritingMultipart() const;
 
     void Answer(IHttpStreamAnswer& stream);
 
@@ -235,5 +196,12 @@ namespace Orthanc
       const std::vector<const void*>& parts,
       const std::vector<size_t>& sizes,
       const std::vector<const std::map<std::string, std::string>*>& headers);
+
+    /**
+     * Contrarily to "Answer()", this method doesn't bufferizes the
+     * stream before sending it, which reduces memory but cannot be
+     * used to handle compression using "Content-Encoding".
+     **/
+    void AnswerWithoutBuffering(IHttpStreamAnswer& stream);
   };
 }
