@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -38,6 +39,14 @@
 #  error Either ORTHANC_ENABLE_MONGOOSE or ORTHANC_ENABLE_CIVETWEB must be set to 1
 #endif
 
+#if !defined(ORTHANC_ENABLE_PUGIXML)
+#  error The macro ORTHANC_ENABLE_PUGIXML must be defined
+#endif
+
+#if ORTHANC_ENABLE_PUGIXML == 1
+#  include "IWebDavBucket.h"
+#endif
+
 
 #include "IIncomingHttpRequestFilter.h"
 
@@ -49,7 +58,6 @@
 
 namespace Orthanc
 {
-  class ChunkStore;
   class OrthancException;
 
   class IHttpExceptionFormatter : public boost::noncopyable
@@ -68,11 +76,19 @@ namespace Orthanc
 
   class ORTHANC_PUBLIC HttpServer : public boost::noncopyable
   {
+  public:
+#if ORTHANC_ENABLE_PUGIXML == 1
+    typedef std::map<std::string, IWebDavBucket*>  WebDavBuckets;
+#endif
+    
   private:
     // http://stackoverflow.com/questions/311166/stdauto-ptr-or-boostshared-ptr-for-pimpl-idiom
     struct PImpl;
     boost::shared_ptr<PImpl> pimpl_;
 
+    class ChunkStore;
+    class MultipartFormDataHandler;
+
     IHttpHandler *handler_;
 
     typedef std::set<std::string> RegisteredUsers;
@@ -80,8 +96,13 @@ namespace Orthanc
 
     bool remoteAllowed_;
     bool authentication_;
+    bool sslVerifyPeers_;
+    std::string trustedClientCertificates_;
     bool ssl_;
     std::string certificate_;
+    unsigned int sslMinimumVersion_;
+    bool sslHasCiphers_;
+    std::string sslCiphers_;
     uint16_t port_;
     IIncomingHttpRequestFilter* filter_;
     bool keepAlive_;
@@ -91,7 +112,11 @@ namespace Orthanc
     unsigned int threadsCount_;
     bool tcpNoDelay_;
     unsigned int requestTimeout_;  // In seconds
-  
+
+#if ORTHANC_ENABLE_PUGIXML == 1
+    WebDavBuckets webDavBuckets_;
+#endif
+    
     bool IsRunning() const;
 
   public:
@@ -101,10 +126,7 @@ namespace Orthanc
 
     void SetPortNumber(uint16_t port);
 
-    uint16_t GetPortNumber() const
-    {
-      return port_;
-    }
+    uint16_t GetPortNumber() const;
 
     void Start();
 
@@ -115,105 +137,88 @@ namespace Orthanc
     void RegisterUser(const char* username,
                       const char* password);
 
-    bool IsAuthenticationEnabled() const
-    {
-      return authentication_;
-    }
+    bool IsAuthenticationEnabled() const;
 
     void SetAuthenticationEnabled(bool enabled);
 
-    bool IsSslEnabled() const
-    {
-      return ssl_;
-    }
+    bool IsSslEnabled() const;
 
     void SetSslEnabled(bool enabled);
 
-    bool IsKeepAliveEnabled() const
-    {
-      return keepAlive_;
-    }
+    void SetSslVerifyPeers(bool enabled);
+
+    // set the minimum accepted version of SSL/TLS protocol according to the CivetWeb table published here:
+    // https://github.com/civetweb/civetweb/blob/master/docs/UserManual.md#ssl_protocol_version-0
+    void SetSslMinimumVersion(unsigned int version);
+
+    void SetSslCiphers(const std::list<std::string>& ciphers);
+    
+    void SetSslTrustedClientCertificates(const char* path);
+
+    bool IsKeepAliveEnabled() const;
 
     void SetKeepAliveEnabled(bool enabled);
 
-    const std::string& GetSslCertificate() const
-    {
-      return certificate_;
-    }
+    const std::string& GetSslCertificate() const;
 
     void SetSslCertificate(const char* path);
 
-    bool IsRemoteAccessAllowed() const
-    {
-      return remoteAllowed_;
-    }
+    bool IsRemoteAccessAllowed() const;
 
     void SetRemoteAccessAllowed(bool allowed);
 
-    bool IsHttpCompressionEnabled() const
-    {
-      return httpCompression_;;
-    }
+    bool IsHttpCompressionEnabled() const;
 
     void SetHttpCompressionEnabled(bool enabled);
 
-    IIncomingHttpRequestFilter* GetIncomingHttpRequestFilter() const
-    {
-      return filter_;
-    }
+    IIncomingHttpRequestFilter* GetIncomingHttpRequestFilter() const;
 
     void SetIncomingHttpRequestFilter(IIncomingHttpRequestFilter& filter);
 
-    ChunkStore& GetChunkStore();
-
     bool IsValidBasicHttpAuthentication(const std::string& basic) const;
 
     void Register(IHttpHandler& handler);
 
-    bool HasHandler() const
-    {
-      return handler_ != NULL;
-    }
+    bool HasHandler() const;
 
     IHttpHandler& GetHandler() const;
 
     void SetHttpExceptionFormatter(IHttpExceptionFormatter& formatter);
 
-    IHttpExceptionFormatter* GetExceptionFormatter()
-    {
-      return exceptionFormatter_;
-    }
+    IHttpExceptionFormatter* GetExceptionFormatter();
 
-    const std::string& GetRealm() const
-    {
-      return realm_;
-    }
+    const std::string& GetRealm() const;
 
-    void SetRealm(const std::string& realm)
-    {
-      realm_ = realm;
-    }
+    void SetRealm(const std::string& realm);
 
     void SetThreadsCount(unsigned int threads);
 
-    unsigned int GetThreadsCount() const
-    {
-      return threadsCount_;
-    }
+    unsigned int GetThreadsCount() const;
 
     // New in Orthanc 1.5.2, not available for Mongoose
     void SetTcpNoDelay(bool tcpNoDelay);
 
-    bool IsTcpNoDelay() const
-    {
-      return tcpNoDelay_;
-    }
+    bool IsTcpNoDelay() const;
 
     void SetRequestTimeout(unsigned int seconds);
 
-    unsigned int GetRequestTimeout() const
-    {
-      return requestTimeout_;
-    }
+    unsigned int GetRequestTimeout() const;
+
+#if ORTHANC_ENABLE_PUGIXML == 1
+    WebDavBuckets& GetWebDavBuckets();
+#endif
+
+#if ORTHANC_ENABLE_PUGIXML == 1
+    void Register(const std::vector<std::string>& root,
+                  IWebDavBucket* bucket); // Takes ownership
+#endif
+
+    ORTHANC_LOCAL
+    void ProcessMultipartFormData(const std::string& remoteIp,
+                                  const std::string& username,
+                                  const UriComponents& uri,
+                                  const std::map<std::string, std::string>& headers,
+                                  const std::string& body,
+                                  const std::string& boundary);
   };
 }
