@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -42,6 +43,13 @@
 
 namespace Orthanc
 {
+  Font::Font() :
+    size_(0),
+    maxHeight_(0)
+  {
+  }
+
+
   Font::~Font()
   {
     for (Characters::iterator it = characters_.begin();
@@ -55,8 +63,7 @@ namespace Orthanc
   void Font::LoadFromMemory(const std::string& font)
   {
     Json::Value v;
-    Json::Reader reader;
-    if (!reader.parse(font, v) ||
+    if (!Toolbox::ReadJson(v, font) ||
         v.type() != Json::objectValue ||
         !v.isMember("Name") ||
         !v.isMember("Size") ||
@@ -141,8 +148,18 @@ namespace Orthanc
   }
 #endif
 
+  const std::string &Font::GetName() const
+  {
+    return name_;
+  }
 
-  static unsigned int MyMin(unsigned int a, 
+  unsigned int Font::GetSize() const
+  {
+    return size_;
+  }
+
+
+  static unsigned int MyMin(unsigned int a,
                             unsigned int b)
   {
     return a < b ? a : b;
