@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -55,6 +56,10 @@ namespace Orthanc
   }
 #endif
 
+  size_t FontRegistry::GetSize() const
+  {
+    return fonts_.size();
+  }
 
   const Font& FontRegistry::GetFont(size_t i) const
   {
@@ -70,7 +75,7 @@ namespace Orthanc
 
   const Font* FontRegistry::FindFont(const std::string& fontName) const
   {
-    for (Fonts::const_iterator it = fonts_.begin(); it != fonts_.end(); it++)
+    for (Fonts::const_iterator it = fonts_.begin(); it != fonts_.end(); ++it)
     {
       if ((*it)->GetName() == fontName)
       {
@@ -80,5 +85,4 @@ namespace Orthanc
 
     return NULL;
   }
-
 }
