@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -42,10 +43,7 @@ namespace Orthanc
     void AddFromFile(const std::string& path);
 #endif
 
-    size_t GetSize() const
-    {
-      return fonts_.size();
-    }
+    size_t GetSize() const;
 
     const Font& GetFont(size_t i) const;
 
