@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -41,7 +42,10 @@ namespace Orthanc
   {
     target.AddChunk("double([ ");
 
-    for (unsigned int y = 0; y < source.GetHeight(); y++)
+    const unsigned int width = source.GetWidth();
+    const unsigned int height = source.GetHeight();
+
+    for (unsigned int y = 0; y < height; y++)
     {
       const PixelType* p = reinterpret_cast<const PixelType*>(source.GetConstRow(y));
 
@@ -51,9 +55,9 @@ namespace Orthanc
         s = "; ";
       }
 
-      s.reserve(source.GetWidth() * 8);
+      s.reserve(width * 8);
 
-      for (unsigned int x = 0; x < source.GetWidth(); x++, p++)
+      for (unsigned int x = 0; x < width; x++, p++)
       {
         s += boost::lexical_cast<std::string>(static_cast<double>(*p)) + " ";
       }
@@ -72,14 +76,17 @@ namespace Orthanc
 
     target.AddChunk("double(permute(reshape([ ");
 
-    for (unsigned int y = 0; y < source.GetHeight(); y++)
+    const unsigned int width = source.GetWidth();
+    const unsigned int height = source.GetHeight();
+
+    for (unsigned int y = 0; y < height; y++)
     {
       const uint8_t* p = reinterpret_cast<const uint8_t*>(source.GetConstRow(y));
       
       std::string s;
-      s.reserve(source.GetWidth() * 3 * 8);
+      s.reserve(width * 3 * 8);
       
-      for (unsigned int x = 0; x < 3 * source.GetWidth(); x++, p++)
+      for (unsigned int x = 0; x < 3 * width; x++, p++)
       {
         s += boost::lexical_cast<std::string>(static_cast<int>(*p)) + " ";
       }
@@ -87,12 +94,61 @@ namespace Orthanc
       target.AddChunk(s);
     }
 
-    target.AddChunk("], [ 3 " + boost::lexical_cast<std::string>(source.GetHeight()) +
-                    " " + boost::lexical_cast<std::string>(source.GetWidth()) + " ]), [ 3 2 1 ]))");
+    target.AddChunk("], [ 3 " + boost::lexical_cast<std::string>(height) +
+                    " " + boost::lexical_cast<std::string>(width) + " ]), [ 3 2 1 ]))");
+  }
+  
+
+  ImageAccessor::ImageAccessor()
+  {
+    AssignEmpty(PixelFormat_Grayscale8);
   }
 
+  ImageAccessor::~ImageAccessor()
+  {
+  }
 
-  void* ImageAccessor::GetBuffer() const
+  bool ImageAccessor::IsReadOnly() const
+  {
+    return readOnly_;
+  }
+
+  PixelFormat ImageAccessor::GetFormat() const
+  {
+    return format_;
+  }
+
+  unsigned int ImageAccessor::GetBytesPerPixel() const
+  {
+    return ::Orthanc::GetBytesPerPixel(format_);
+  }
+
+  unsigned int ImageAccessor::GetWidth() const
+  {
+    return width_;
+  }
+
+  unsigned int ImageAccessor::GetHeight() const
+  {
+    return height_;
+  }
+
+  unsigned int ImageAccessor::GetPitch() const
+  {
+    return pitch_;
+  }
+
+  unsigned int ImageAccessor::GetSize() const
+  {
+    return GetHeight() * GetPitch();
+  }
+
+  const void *ImageAccessor::GetConstBuffer() const
+  {
+    return buffer_;
+  }
+
+  void* ImageAccessor::GetBuffer()
   {
     if (readOnly_)
     {
@@ -117,7 +173,7 @@ namespace Orthanc
   }
 
 
-  void* ImageAccessor::GetRow(unsigned int y) const
+  void* ImageAccessor::GetRow(unsigned int y)
   {
     if (readOnly_)
     {
@@ -166,6 +222,11 @@ namespace Orthanc
     }
   }
 
+  void ImageAccessor::GetReadOnlyAccessor(ImageAccessor &target) const
+  {
+    target.AssignReadOnly(format_, width_, height_, pitch_, buffer_);
+  }
+
 
   void ImageAccessor::AssignWritable(PixelFormat format,
                                      unsigned int width,
@@ -293,4 +354,17 @@ namespace Orthanc
 
     format_ = format;
   }
+
+
+#if ORTHANC_BUILDING_FRAMEWORK_LIBRARY == 1
+  void* ImageAccessor::GetBuffer() const
+  {
+    return const_cast<ImageAccessor&>(*this).GetBuffer();
+  }
+
+  void* ImageAccessor::GetRow(unsigned int y) const
+  {
+    return const_cast<ImageAccessor&>(*this).GetRow(y);
+  }
+#endif
 }
