@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -23,7 +24,7 @@
 #pragma once
 
 #include "IJob.h"
-#include "Operations/JobOperationValue.h"
+#include "Operations/IJobOperationValue.h"
 #include "Operations/IJobOperation.h"
 
 #include <vector>
@@ -41,6 +42,6 @@ namespace Orthanc
 
     virtual IJobOperation* UnserializeOperation(const Json::Value& value) = 0;
 
-    virtual JobOperationValue* UnserializeValue(const Json::Value& value) = 0;
+    virtual IJobOperationValue* UnserializeValue(const Json::Value& value) = 0;
   };
 }
