@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,6 +26,8 @@
 #include "../IJob.h"
 #include "IJobOperation.h"
 
+#include "../../Compatibility.h"  // For ORTHANC_OVERRIDE
+
 #include <boost/thread/mutex.hpp>
 #include <boost/thread/condition_variable.hpp>
 
@@ -82,52 +85,40 @@ namespace Orthanc
       SequenceOfOperationsJob&   that_;
       boost::mutex::scoped_lock  lock_;
 
+#if ORTHANC_BUILDING_FRAMEWORK_LIBRARY == 1
+      ORTHANC_DEPRECATED(void AddInput(size_t index,
+                                       const JobOperationValue& value));
+#endif
+      
     public:
-      Lock(SequenceOfOperationsJob& that) :
-        that_(that),
-        lock_(that.mutex_)
-      {
-      }
+      explicit Lock(SequenceOfOperationsJob& that);
 
-      bool IsDone() const
-      {
-        return that_.done_;
-      }
+      bool IsDone() const;
 
       void SetTrailingOperationTimeout(unsigned int timeout);
       
       size_t AddOperation(IJobOperation* operation);
 
-      size_t GetOperationsCount() const
-      {
-        return that_.operations_.size();
-      }
+      size_t GetOperationsCount() const;
 
       void AddInput(size_t index,
-                    const JobOperationValue& value);
+                    const IJobOperationValue& value);
       
       void Connect(size_t input,
                    size_t output);
     };
 
-    virtual void Start() ORTHANC_OVERRIDE
-    {
-    }
+    virtual void Start() ORTHANC_OVERRIDE;
 
     virtual JobStepResult Step(const std::string& jobId) ORTHANC_OVERRIDE;
 
     virtual void Reset() ORTHANC_OVERRIDE;
 
-    virtual void Stop(JobStopReason reason) ORTHANC_OVERRIDE
-    {
-    }
+    virtual void Stop(JobStopReason reason) ORTHANC_OVERRIDE;
 
     virtual float GetProgress() ORTHANC_OVERRIDE;
 
-    virtual void GetJobType(std::string& target) ORTHANC_OVERRIDE
-    {
-      target = "SequenceOfOperations";
-    }
+    virtual void GetJobType(std::string& target) ORTHANC_OVERRIDE;
 
     virtual void GetPublicContent(Json::Value& value) ORTHANC_OVERRIDE;
 
@@ -135,14 +126,9 @@ namespace Orthanc
 
     virtual bool GetOutput(std::string& output,
                            MimeType& mime,
-                           const std::string& key) ORTHANC_OVERRIDE
-    {
-      return false;
-    }
+                           std::string& filename,
+                           const std::string& key) ORTHANC_OVERRIDE;
 
-    void AwakeTrailingSleep()
-    {
-      operationAdded_.notify_one();
-    }
+    void AwakeTrailingSleep();
   };
 }
