@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -94,9 +95,10 @@ namespace Orthanc
     }
 
   public:
-    Item(MetricsType type) :
-    type_(type),
-    hasValue_(false)
+    explicit Item(MetricsType type) :
+      type_(type),
+      hasValue_(false),
+      value_(0)
     {
     }
 
@@ -174,6 +176,11 @@ namespace Orthanc
     }
   }
 
+  bool MetricsRegistry::IsEnabled() const
+  {
+    return enabled_;
+  }
+
 
   void MetricsRegistry::SetEnabled(bool enabled)
   {
@@ -207,7 +214,6 @@ namespace Orthanc
     }    
   }
 
-
   void MetricsRegistry::SetValueInternal(const std::string& name,
                                          float value,
                                          MetricsType type)
@@ -229,6 +235,29 @@ namespace Orthanc
     }
   }
 
+  MetricsRegistry::MetricsRegistry() :
+    enabled_(true)
+  {
+  }
+
+
+  void MetricsRegistry::SetValue(const std::string &name,
+                                 float value,
+                                 MetricsType type)
+  {
+    // Inlining to avoid loosing time if metrics are disabled
+    if (enabled_)
+    {
+      SetValueInternal(name, value, type);
+    }
+  }
+
+
+  void MetricsRegistry::SetValue(const std::string &name, float value)
+  {
+    SetValue(name, value, MetricsType_Default);
+  }
+
 
   MetricsType MetricsRegistry::GetMetricsType(const std::string& name)
   {
@@ -285,6 +314,15 @@ namespace Orthanc
   }
 
 
+  MetricsRegistry::SharedMetrics::SharedMetrics(MetricsRegistry &registry,
+                                                const std::string &name,
+                                                MetricsType type) :
+    registry_(registry),
+    name_(name),
+    value_(0)
+  {
+  }
+
   void MetricsRegistry::SharedMetrics::Add(float delta)
   {
     boost::mutex::scoped_lock lock(mutex_);
@@ -293,6 +331,18 @@ namespace Orthanc
   }
 
 
+  MetricsRegistry::ActiveCounter::ActiveCounter(MetricsRegistry::SharedMetrics &metrics) :
+    metrics_(metrics)
+  {
+    metrics_.Add(1);
+  }
+
+  MetricsRegistry::ActiveCounter::~ActiveCounter()
+  {
+    metrics_.Add(-1);
+  }
+
+
   void  MetricsRegistry::Timer::Start()
   {
     if (registry_.IsEnabled())
@@ -307,13 +357,34 @@ namespace Orthanc
   }
 
 
+  MetricsRegistry::Timer::Timer(MetricsRegistry &registry,
+                                const std::string &name) :
+    registry_(registry),
+    name_(name),
+    type_(MetricsType_MaxOver10Seconds)
+  {
+    Start();
+  }
+
+
+  MetricsRegistry::Timer::Timer(MetricsRegistry &registry,
+                                const std::string &name,
+                                MetricsType type) :
+    registry_(registry),
+    name_(name),
+    type_(type)
+  {
+    Start();
+  }
+
+
   MetricsRegistry::Timer::~Timer()
   {
     if (active_)
-    {   
+    {
       boost::posix_time::time_duration diff = GetNow() - start_;
       registry_.SetValue(
-        name_, static_cast<float>(diff.total_milliseconds()), type_);
+            name_, static_cast<float>(diff.total_milliseconds()), type_);
     }
   }
 }
