@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -24,6 +25,7 @@
 
 #include "RestApiHierarchy.h"
 #include "../Compatibility.h"
+#include "../HttpServer/IHttpHandler.h"
 
 #include <list>
 
@@ -43,10 +45,7 @@ namespace Orthanc
                                             const char* username,
                                             HttpMethod method,
                                             const UriComponents& uri,
-                                            const Arguments& headers)
-    {
-      return false;
-    }
+                                            const HttpToolbox::Arguments& headers) ORTHANC_OVERRIDE;
 
     virtual bool Handle(HttpOutput& output,
                         RequestOrigin origin,
@@ -54,10 +53,10 @@ namespace Orthanc
                         const char* username,
                         HttpMethod method,
                         const UriComponents& uri,
-                        const Arguments& headers,
-                        const GetArguments& getArguments,
+                        const HttpToolbox::Arguments& headers,
+                        const HttpToolbox::GetArguments& getArguments,
                         const void* bodyData,
-                        size_t bodySize);
+                        size_t bodySize) ORTHANC_OVERRIDE;
 
     void Register(const std::string& path,
                   RestApiGetCall::Handler handler);
@@ -70,5 +69,10 @@ namespace Orthanc
 
     void Register(const std::string& path,
                   RestApiDeleteCall::Handler handler);
+
+    void GenerateOpenApiDocumentation(Json::Value& target);
+
+    void GenerateReStructuredTextCheatSheet(std::string& target,
+                                            const std::string& openApiUrl);
   };
 }
