@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2012-2016 Sebastien Jodogne <s.jodogne@orthanc-labs.com>,
  * Medical Physics Department, CHU of Liege, Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * Copyright (c) 2012 The Chromium Authors. All rights reserved.
  *
@@ -104,10 +106,7 @@ namespace Orthanc
 
       bool Execute(const char* sql);
 
-      bool Execute(const std::string& sql)
-      {
-        return Execute(sql.c_str());
-      }
+      bool Execute(const std::string& sql);
 
       void FlushToDisk();
 
@@ -155,15 +154,9 @@ namespace Orthanc
 
       int ExecuteAndReturnErrorCode(const char* sql);
     
-      bool HasCachedStatement(const StatementId& id) const
-      {
-        return cachedStatements_.find(id) != cachedStatements_.end();
-      }
+      bool HasCachedStatement(const StatementId& id) const;
 
-      int GetTransactionNesting() const
-      {
-        return transactionNesting_;
-      }
+      int GetTransactionNesting() const;
 
       // Transactions --------------------------------------------------------------
 
