@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -33,6 +34,7 @@
 #endif
 
 #include <boost/noncopyable.hpp>
+#include <stdint.h>
 #include <string>
 
 namespace Orthanc
@@ -50,15 +52,19 @@ namespace Orthanc
 
     ~TemporaryFile();
 
-    const std::string& GetPath() const
-    {
-      return path_;
-    }
+    const std::string& GetPath() const;
 
     void Write(const std::string& content);
 
     void Read(std::string& content) const;
 
     void Touch();
+
+    uint64_t GetFileSize() const;
+
+    void ReadRange(std::string& content,
+                   uint64_t start,
+                   uint64_t end,
+                   bool throwIfOverflow) const;
   };
 }
