@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -92,6 +93,11 @@
 #  include <openssl/err.h>
 #  include <openssl/evp.h>
 #  include <openssl/ssl.h>
+
+#  if OPENSSL_VERSION_NUMBER < 0x30000000L
+#    warning You are linking Orthanc against OpenSSL 1.x, whose license is incompatible with the GPLv3+ used by Orthanc >= 1.10.0. Please update to OpenSSL 3.x, that uses the Apache 2 license.
+#  endif
+
 #endif
 
 
@@ -1703,6 +1709,7 @@ namespace Orthanc
    **/
   void Toolbox::InitializeOpenSsl()
   {
+    LOG(INFO) << "OpenSSL is disabled";
   }
   
   void Toolbox::FinalizeOpenSsl()
@@ -1717,6 +1724,8 @@ namespace Orthanc
    **/
   void Toolbox::InitializeOpenSsl()
   {
+    LOG(INFO) << "OpenSSL version: " << OPENSSL_VERSION_TEXT;
+    
     // https://wiki.openssl.org/index.php/Library_Initialization
     SSL_library_init();
     SSL_load_error_strings();
@@ -1753,6 +1762,7 @@ namespace Orthanc
    **/
   void Toolbox::InitializeOpenSsl()
   {
+    LOG(INFO) << "OpenSSL version: " << OPENSSL_VERSION_TEXT;
   }
 
   void Toolbox::FinalizeOpenSsl()
