@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -30,6 +31,7 @@
 
 #include <map>
 #include <set>
+#include <stdint.h>
 #include <string>
 #include <json/value.h>
 
@@ -50,6 +52,7 @@ namespace Orthanc
     bool         pkcs11Enabled_;
     Dictionary   headers_;
     Dictionary   userProperties_;
+    unsigned int timeout_;
 
     void FromSimpleFormat(const Json::Value& peer);
 
@@ -129,5 +132,12 @@ namespace Orthanc
 #endif
 
     void FormatPublic(Json::Value& target) const;
+
+    // Setting it to "0" will use "HttpClient::SetDefaultTimeout()"
+    void SetTimeout(uint32_t seconds);
+
+    uint32_t GetTimeout() const;
+
+    bool HasTimeout() const;    
   };
 }
