@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -36,9 +37,12 @@
 #include "../Sources/Images/PngWriter.h"
 #include "../Sources/Images/PamReader.h"
 #include "../Sources/Images/PamWriter.h"
-#include "../Sources/SystemToolbox.h"
 #include "../Sources/Toolbox.h"
-#include "../Sources/TemporaryFile.h"
+
+#if ORTHANC_SANDBOXED != 1
+#  include "../Sources/SystemToolbox.h"
+#  include "../Sources/TemporaryFile.h"
+#endif
 
 #include <stdint.h>
 
@@ -65,14 +69,21 @@ TEST(PngWriter, ColorPattern)
   Orthanc::ImageAccessor accessor;
   accessor.AssignReadOnly(Orthanc::PixelFormat_RGB24, width, height, pitch, &image[0]);
 
-  w.WriteToFile("UnitTestsResults/ColorPattern.png", accessor);
+  std::string f;
 
-  std::string f, md5;
+#if ORTHANC_SANDBOXED == 1
+  Orthanc::IImageWriter::WriteToMemory(w, f, accessor);
+#else
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/ColorPattern.png", accessor);
   Orthanc::SystemToolbox::ReadFile(f, "UnitTestsResults/ColorPattern.png");
+#endif
+
+  std::string md5;
   Orthanc::Toolbox::ComputeMD5(md5, f);
   ASSERT_EQ("604e785f53c99cae6ea4584870b2c41d", md5);
 }
 
+
 TEST(PngWriter, Gray8Pattern)
 {
   Orthanc::PngWriter w;
@@ -93,10 +104,16 @@ TEST(PngWriter, Gray8Pattern)
   Orthanc::ImageAccessor accessor;
   accessor.AssignReadOnly(Orthanc::PixelFormat_Grayscale8, width, height, pitch, &image[0]);
 
-  w.WriteToFile("UnitTestsResults/Gray8Pattern.png", accessor);
-
-  std::string f, md5;
+  std::string f;
+  
+#if ORTHANC_SANDBOXED == 1
+  Orthanc::IImageWriter::WriteToMemory(w, f, accessor);
+#else
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/Gray8Pattern.png", accessor);
   Orthanc::SystemToolbox::ReadFile(f, "UnitTestsResults/Gray8Pattern.png");
+#endif
+
+  std::string md5;
   Orthanc::Toolbox::ComputeMD5(md5, f);
   ASSERT_EQ("5a9b98bea3d0a6d983980cc38bfbcdb3", md5);
 }
@@ -122,10 +139,17 @@ TEST(PngWriter, Gray16Pattern)
 
   Orthanc::ImageAccessor accessor;
   accessor.AssignReadOnly(Orthanc::PixelFormat_Grayscale16, width, height, pitch, &image[0]);
-  w.WriteToFile("UnitTestsResults/Gray16Pattern.png", accessor);
 
-  std::string f, md5;
+  std::string f;
+  
+#if ORTHANC_SANDBOXED == 1
+  Orthanc::IImageWriter::WriteToMemory(w, f, accessor);
+#else
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/Gray16Pattern.png", accessor);
   Orthanc::SystemToolbox::ReadFile(f, "UnitTestsResults/Gray16Pattern.png");
+#endif
+
+  std::string md5;
   Orthanc::Toolbox::ComputeMD5(md5, f);
   ASSERT_EQ("0785866a08bf0a02d2eeff87f658571c", md5);
 }
@@ -153,7 +177,7 @@ TEST(PngWriter, EndToEnd)
   accessor.AssignReadOnly(Orthanc::PixelFormat_Grayscale16, width, height, pitch, &image[0]);
 
   std::string s;
-  w.WriteToMemory(s, accessor);
+  Orthanc::IImageWriter::WriteToMemory(w, s, accessor);
 
   {
     Orthanc::PngReader r;
@@ -175,6 +199,7 @@ TEST(PngWriter, EndToEnd)
     }
   }
 
+#if ORTHANC_SANDBOXED != 1
   {
     Orthanc::TemporaryFile tmp;
     tmp.Write(s);
@@ -197,6 +222,7 @@ TEST(PngWriter, EndToEnd)
       }
     }
   }
+#endif
 }
 
 
@@ -218,34 +244,51 @@ TEST(JpegWriter, Basic)
     }
 
     Orthanc::JpegWriter w;
-    w.WriteToFile("UnitTestsResults/hello.jpg", img);
+    Orthanc::IImageWriter::WriteToMemory(w, s, img);
 
-    w.WriteToMemory(s, img);
+#if ORTHANC_SANDBOXED != 1
+    Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/hello.jpg", img);
     Orthanc::SystemToolbox::WriteFile(s, "UnitTestsResults/hello2.jpg");
 
     std::string t;
     Orthanc::SystemToolbox::ReadFile(t, "UnitTestsResults/hello.jpg");
     ASSERT_EQ(s.size(), t.size());
     ASSERT_EQ(0, memcmp(s.c_str(), t.c_str(), s.size()));
+#endif
   }
 
   {
-    Orthanc::JpegReader r1, r2;
-    r1.ReadFromFile("UnitTestsResults/hello.jpg");
+    Orthanc::JpegReader r1;
+    r1.ReadFromMemory(s);
     ASSERT_EQ(16u, r1.GetWidth());
     ASSERT_EQ(16u, r1.GetHeight());
 
-    r2.ReadFromMemory(s);
+#if ORTHANC_SANDBOXED != 1
+    Orthanc::JpegReader r2;
+    r2.ReadFromFile("UnitTestsResults/hello.jpg");
     ASSERT_EQ(16u, r2.GetWidth());
     ASSERT_EQ(16u, r2.GetHeight());
+#endif
 
+    unsigned int value = 0;
     for (unsigned int y = 0; y < r1.GetHeight(); y++)
     {
       const uint8_t* p1 = reinterpret_cast<const uint8_t*>(r1.GetConstRow(y));
+#if ORTHANC_SANDBOXED != 1
       const uint8_t* p2 = reinterpret_cast<const uint8_t*>(r2.GetConstRow(y));
-      for (unsigned int x = 0; x < r1.GetWidth(); x++)
+#endif
+      for (unsigned int x = 0; x < r1.GetWidth(); x++, value++)
       {
+        ASSERT_TRUE(*p1 == value ||
+                    *p1 == value - 1 ||
+                    *p1 == value + 1);  // Be tolerant to differences of +-1
+
+#if ORTHANC_SANDBOXED != 1
         ASSERT_EQ(*p1, *p2);
+        p2++;
+#endif
+
+        p1++;
       }
     }
   }
@@ -274,10 +317,16 @@ TEST(PamWriter, ColorPattern)
   Orthanc::ImageAccessor accessor;
   accessor.AssignReadOnly(Orthanc::PixelFormat_RGB24, width, height, pitch, &image[0]);
 
-  w.WriteToFile("UnitTestsResults/ColorPattern.pam", accessor);
+  std::string f;
 
-  std::string f, md5;
+#if ORTHANC_SANDBOXED == 1
+  Orthanc::IImageWriter::WriteToMemory(w, f, accessor);
+#else
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/ColorPattern.pam", accessor);
   Orthanc::SystemToolbox::ReadFile(f, "UnitTestsResults/ColorPattern.pam");
+#endif
+
+  std::string md5;
   Orthanc::Toolbox::ComputeMD5(md5, f);
   ASSERT_EQ("81a3441754e88969ebbe53e69891e841", md5);
 }
@@ -302,10 +351,16 @@ TEST(PamWriter, Gray8Pattern)
   Orthanc::ImageAccessor accessor;
   accessor.AssignReadOnly(Orthanc::PixelFormat_Grayscale8, width, height, pitch, &image[0]);
 
-  w.WriteToFile("UnitTestsResults/Gray8Pattern.pam", accessor);
+  std::string f;
 
-  std::string f, md5;
+#if ORTHANC_SANDBOXED == 1
+  Orthanc::IImageWriter::WriteToMemory(w, f, accessor);
+#else
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/Gray8Pattern.pam", accessor);
   Orthanc::SystemToolbox::ReadFile(f, "UnitTestsResults/Gray8Pattern.pam");
+#endif
+  
+  std::string md5;
   Orthanc::Toolbox::ComputeMD5(md5, f);
   ASSERT_EQ("7873c408d26a9d11dd1c1de5e69cc0a3", md5);
 }
@@ -331,10 +386,17 @@ TEST(PamWriter, Gray16Pattern)
 
   Orthanc::ImageAccessor accessor;
   accessor.AssignReadOnly(Orthanc::PixelFormat_Grayscale16, width, height, pitch, &image[0]);
-  w.WriteToFile("UnitTestsResults/Gray16Pattern.pam", accessor);
 
-  std::string f, md5;
+  std::string f;
+
+#if ORTHANC_SANDBOXED == 1
+  Orthanc::IImageWriter::WriteToMemory(w, f, accessor);
+#else
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/Gray16Pattern.pam", accessor);
   Orthanc::SystemToolbox::ReadFile(f, "UnitTestsResults/Gray16Pattern.pam");
+#endif
+
+  std::string md5;
   Orthanc::Toolbox::ComputeMD5(md5, f);
   ASSERT_EQ("b268772bf28f3b2b8520ff21c5e3dcb6", md5);
 }
@@ -362,10 +424,10 @@ TEST(PamWriter, EndToEnd)
   accessor.AssignReadOnly(Orthanc::PixelFormat_Grayscale16, width, height, pitch, &image[0]);
 
   std::string s;
-  w.WriteToMemory(s, accessor);
+  Orthanc::IImageWriter::WriteToMemory(w, s, accessor);
 
   {
-    Orthanc::PamReader r;
+    Orthanc::PamReader r(true);
     r.ReadFromMemory(s);
 
     ASSERT_EQ(r.GetFormat(), Orthanc::PixelFormat_Grayscale16);
@@ -407,11 +469,12 @@ TEST(PamWriter, EndToEnd)
     }
   }
 
+#if ORTHANC_SANDBOXED != 1
   {
     Orthanc::TemporaryFile tmp;
     tmp.Write(s);
 
-    Orthanc::PamReader r2;
+    Orthanc::PamReader r2(true);
     r2.ReadFromFile(tmp.GetPath());
 
     ASSERT_EQ(r2.GetFormat(), Orthanc::PixelFormat_Grayscale16);
@@ -430,7 +493,9 @@ TEST(PamWriter, EndToEnd)
       }
     }
   }
+#endif
 
+#if ORTHANC_SANDBOXED != 1
   {
     Orthanc::TemporaryFile tmp;
     tmp.Write(s);
@@ -455,5 +520,44 @@ TEST(PamWriter, EndToEnd)
       }
     }
   }
+#endif
+}
 
+
+TEST(PngWriter, Gray16Then8)
+{
+  Orthanc::Image image16(Orthanc::PixelFormat_Grayscale16, 32, 32, false);
+  Orthanc::Image image8(Orthanc::PixelFormat_Grayscale8, 32, 32, false);
+
+  memset(image16.GetBuffer(), 0, image16.GetHeight() * image16.GetPitch());
+  memset(image8.GetBuffer(), 0, image8.GetHeight() * image8.GetPitch());
+
+  {
+    Orthanc::PamWriter w;
+    std::string s;
+    Orthanc::IImageWriter::WriteToMemory(w, s, image16);
+    Orthanc::IImageWriter::WriteToMemory(w, s, image8);  // No problem here
+  }
+
+  {
+    Orthanc::PamWriter w;
+    std::string s;
+    Orthanc::IImageWriter::WriteToMemory(w, s, image8);
+    Orthanc::IImageWriter::WriteToMemory(w, s, image16);  // No problem here
+  }
+
+  {
+    Orthanc::PngWriter w;
+    std::string s;
+    Orthanc::IImageWriter::WriteToMemory(w, s, image8);
+    Orthanc::IImageWriter::WriteToMemory(w, s, image16);  // No problem here
+  }  
+
+  {
+    // The following call leads to "Invalid read of size 1" in Orthanc <= 1.9.2
+    Orthanc::PngWriter w;
+    std::string s;
+    Orthanc::IImageWriter::WriteToMemory(w, s, image16);
+    Orthanc::IImageWriter::WriteToMemory(w, s, image8);  // Problem here
+  }  
 }
