@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -461,7 +450,8 @@ namespace Orthanc
 
     bool ExpandResource(Json::Value& target,
                         const std::string& publicId,
-                        ResourceType level);
+                        ResourceType level,
+                        DicomToJsonFormat format);
 
     void GetAllMetadata(std::map<MetadataType, std::string>& target,
                         const std::string& publicId,
@@ -565,7 +555,7 @@ namespace Orthanc
                               ResourceType queryLevel,
                               size_t limit);
 
-    bool DeleteResource(Json::Value& target /* out */,
+    bool DeleteResource(Json::Value& remainingAncestor /* out */,
                         const std::string& uuid,
                         ResourceType expectedType);
 
@@ -580,7 +570,8 @@ namespace Orthanc
                      MetadataType type,
                      const std::string& value,
                      bool hasOldRevision,
-                     int64_t oldRevision);
+                     int64_t oldRevision,
+                     const std::string& oldMD5);
 
     // Same as "SetMetadata()", but doesn't care about revisions
     void OverwriteMetadata(const std::string& publicId,
@@ -590,7 +581,8 @@ namespace Orthanc
     bool DeleteMetadata(const std::string& publicId,
                         MetadataType type,
                         bool hasRevision,
-                        int64_t revision);
+                        int64_t revision,
+                        const std::string& md5);
 
     uint64_t IncrementGlobalSequence(GlobalProperty sequence,
                                      bool shared);
@@ -606,7 +598,8 @@ namespace Orthanc
     bool DeleteAttachment(const std::string& publicId,
                           FileContentType type,
                           bool hasRevision,
-                          int64_t revision);
+                          int64_t revision,
+                          const std::string& md5);
 
     void LogChange(int64_t internalId,
                    ChangeType changeType,
@@ -634,6 +627,7 @@ namespace Orthanc
                               uint64_t maximumStorageSize,
                               unsigned int maximumPatients,
                               bool hasOldRevision,
-                              int64_t oldRevision);
+                              int64_t oldRevision,
+                              const std::string& oldMd5);
   };
 }
