@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -200,7 +189,7 @@ namespace Orthanc
              it = instances.begin(); it != instances.end(); ++it)
       {
         std::string value;
-        if (context.LookupOrReconstructMetadata(value, *it, MetadataType_Instance_SopClassUid))
+        if (context.LookupOrReconstructMetadata(value, *it, ResourceType_Instance, MetadataType_Instance_SopClassUid))
         {
           values.insert(value);
         }
@@ -368,7 +357,7 @@ namespace Orthanc
     if (result.GetSize() == 0 &&
         sequencesToReturn.empty())
     {
-      LOG(WARNING) << "The C-FIND request does not return any DICOM tag";
+      CLOG(WARNING, DICOM) << "The C-FIND request does not return any DICOM tag";
     }
     else if (sequencesToReturn.empty())
     {
@@ -376,7 +365,7 @@ namespace Orthanc
     }
     else if (dicomAsJson == NULL)
     {
-      LOG(WARNING) << "C-FIND query requesting a sequence, but reading JSON from disk is disabled";
+      CLOG(WARNING, DICOM) << "C-FIND query requesting a sequence, but reading JSON from disk is disabled";
       answers.Add(result);
     }
     else
@@ -438,7 +427,7 @@ namespace Orthanc
   {
     // Whatever the manufacturer, remove the GenericGroupLength tags
     // http://dicom.nema.org/medical/dicom/current/output/chtml/part05/sect_7.2.html
-    // https://bitbucket.org/sjodogne/orthanc/issues/31/
+    // https://bugs.orthanc-server.com/show_bug.cgi?id=31
     if (tag.GetElement() == 0x0000)
     {
       return false;
@@ -538,7 +527,7 @@ namespace Orthanc
       }
     }
 
-    virtual bool IsDicomAsJsonNeeded() const
+    virtual bool IsDicomAsJsonNeeded() const ORTHANC_OVERRIDE
     {
       // Ask the "DICOM-as-JSON" attachment only if sequences are to
       // be returned OR if "query_" contains non-main DICOM tags!
@@ -564,7 +553,7 @@ namespace Orthanc
               !withoutSpecialTags.HasOnlyMainDicomTags());
     }
       
-    virtual void MarkAsComplete()
+    virtual void MarkAsComplete() ORTHANC_OVERRIDE
     {
       answers_.SetComplete(true);
     }
@@ -572,7 +561,7 @@ namespace Orthanc
     virtual void Visit(const std::string& publicId,
                        const std::string& instanceId,
                        const DicomMap& mainDicomTags,
-                       const Json::Value* dicomAsJson) 
+                       const Json::Value* dicomAsJson) ORTHANC_OVERRIDE
     {
       std::unique_ptr<DicomMap> counters(ComputeCounters(context_, instanceId, level_, query_));
 
@@ -592,6 +581,36 @@ namespace Orthanc
   {
     MetricsRegistry::Timer timer(context_.GetMetricsRegistry(), "orthanc_find_scp_duration_ms");
 
+
+    /**
+     * Deal with global configuration
+     **/
+
+    bool caseSensitivePN;
+
+    {
+      OrthancConfiguration::ReaderLock lock;
+      caseSensitivePN = lock.GetConfiguration().GetBooleanParameter("CaseSensitivePN", false);
+
+      RemoteModalityParameters remote;
+      if (!lock.GetConfiguration().LookupDicomModalityUsingAETitle(remote, remoteAet))
+      {
+        if (lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowFind", false))
+        {
+          CLOG(INFO, DICOM) << "C-FIND: Allowing SCU request from unknown modality with AET: " << remoteAet;
+        }
+        else
+        {
+          // This should never happen, given the test at bottom of
+          // "OrthancApplicationEntityFilter::IsAllowedRequest()"
+          throw OrthancException(ErrorCode_InexistentItem,
+                                 "C-FIND: Rejecting SCU request from unknown modality with AET: " + remoteAet);
+        }
+      }
+    }
+
+
+
     /**
      * Possibly apply the user-supplied Lua filter.
      **/
@@ -631,24 +650,24 @@ namespace Orthanc
 
 
     DicomArray query(*filteredInput);
-    LOG(INFO) << "DICOM C-Find request at level: " << EnumerationToString(level);
+    CLOG(INFO, DICOM) << "DICOM C-Find request at level: " << EnumerationToString(level);
 
     for (size_t i = 0; i < query.GetSize(); i++)
     {
       if (!query.GetElement(i).GetValue().IsNull())
       {
-        LOG(INFO) << "  " << query.GetElement(i).GetTag()
-                  << "  " << FromDcmtkBridge::GetTagName(query.GetElement(i))
-                  << " = " << query.GetElement(i).GetValue().GetContent();
+        CLOG(INFO, DICOM) << "  (" << query.GetElement(i).GetTag().Format()
+                          << ")  " << FromDcmtkBridge::GetTagName(query.GetElement(i))
+                          << " = " << context_.GetDeidentifiedContent(query.GetElement(i));
       }
     }
 
     for (std::list<DicomTag>::const_iterator it = sequencesToReturn.begin();
          it != sequencesToReturn.end(); ++it)
     {
-      LOG(INFO) << "  (" << it->Format()
-                << ")  " << FromDcmtkBridge::GetTagName(*it, "")
-                << " : sequence tag whose content will be copied";
+      CLOG(INFO, DICOM) << "  (" << it->Format()
+                        << ")  " << FromDcmtkBridge::GetTagName(*it, "")
+                        << " : sequence tag whose content will be copied";
     }
 
     // collect the private creators from the query itself
@@ -668,13 +687,6 @@ namespace Orthanc
 
     DatabaseLookup lookup;
 
-    bool caseSensitivePN;
-
-    {
-      OrthancConfiguration::ReaderLock lock;
-      caseSensitivePN = lock.GetConfiguration().GetBooleanParameter("CaseSensitivePN", false);
-    }
-
     for (size_t i = 0; i < query.GetSize(); i++)
     {
       const DicomElement& element = query.GetElement(i);
@@ -710,8 +722,9 @@ namespace Orthanc
       }
       else
       {
-        LOG(INFO) << "Because of a patch for the manufacturer of the remote modality, " 
-                  << "ignoring constraint on tag (" << tag.Format() << ") " << FromDcmtkBridge::GetTagName(element);
+        CLOG(INFO, DICOM) << "Because of a patch for the manufacturer of the remote modality, " 
+                          << "ignoring constraint on tag (" << tag.Format() << ") "
+                          << FromDcmtkBridge::GetTagName(element);
       }
     }
 
