@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,6 +35,29 @@
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
+#define INFO_SUBSEQUENCES \
+  "Starting with Orthanc 1.9.4, paths to subsequences can be provided using the "\
+  "same syntax as the `dcmodify` command-line tool (wildcards are supported as well)."
+
+
+static const char* const CONTENT = "Content";
+static const char* const FORCE = "Force";
+static const char* const INSTANCES = "Instances";
+static const char* const INTERPRET_BINARY_TAGS = "InterpretBinaryTags";
+static const char* const KEEP = "Keep";
+static const char* const KEEP_PRIVATE_TAGS = "KeepPrivateTags";
+static const char* const KEEP_SOURCE = "KeepSource";
+static const char* const LEVEL = "Level";
+static const char* const PARENT = "Parent";
+static const char* const PRIVATE_CREATOR = "PrivateCreator";
+static const char* const REMOVE = "Remove";
+static const char* const REPLACE = "Replace";
+static const char* const RESOURCES = "Resources";
+static const char* const SERIES = "Series";
+static const char* const TAGS = "Tags";
+static const char* const TRANSCODE = "Transcode";
+
+
 namespace Orthanc
 {
   // Modification of DICOM instances ------------------------------------------
@@ -58,28 +70,40 @@ namespace Orthanc
   }
 
 
+  static void DocumentKeepSource(RestApiPostCall& call)
+  {
+    call.GetDocumentation()
+      .SetRequestField(KEEP_SOURCE, RestApiCallDocumentation::Type_Boolean,
+                       "If set to `false`, instructs Orthanc to the remove original resources. "
+                       "By default, the original resources are kept in Orthanc.", false);
+  }
+
+
   static void DocumentModifyOptions(RestApiPostCall& call)
   {
     // Check out "DicomModification::ParseModifyRequest()"
     call.GetDocumentation()
-      .SetRequestField("Transcode", RestApiCallDocumentation::Type_String,
+      .SetRequestField(TRANSCODE, RestApiCallDocumentation::Type_String,
                        "Transcode the DICOM instances to the provided DICOM transfer syntax: "
                        "https://book.orthanc-server.com/faq/transcoding.html", false)
-      .SetRequestField("Force", RestApiCallDocumentation::Type_Boolean,
+      .SetRequestField(FORCE, RestApiCallDocumentation::Type_Boolean,
                        "Allow the modification of tags related to DICOM identifiers, at the risk of "
                        "breaking the DICOM model of the real world", false)
       .SetRequestField("RemovePrivateTags", RestApiCallDocumentation::Type_Boolean,
                        "Remove the private tags from the DICOM instances (defaults to `false`)", false)
-      .SetRequestField("Replace", RestApiCallDocumentation::Type_JsonObject,
-                       "Associative array to change the value of some DICOM tags in the DICOM instances", false)
-      .SetRequestField("Remove", RestApiCallDocumentation::Type_JsonListOfStrings,
-                       "List of tags that must be removed from the DICOM instances", false)
-      .SetRequestField("Keep", RestApiCallDocumentation::Type_JsonListOfStrings,
+      .SetRequestField(REPLACE, RestApiCallDocumentation::Type_JsonObject,
+                       "Associative array to change the value of some DICOM tags in the DICOM instances. " INFO_SUBSEQUENCES, false)
+      .SetRequestField(REMOVE, RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "List of tags that must be removed from the DICOM instances. " INFO_SUBSEQUENCES, false)
+      .SetRequestField(KEEP, RestApiCallDocumentation::Type_JsonListOfStrings,
                        "Keep the original value of the specified tags, to be chosen among the `StudyInstanceUID`, "
                        "`SeriesInstanceUID` and `SOPInstanceUID` tags. Avoid this feature as much as possible, "
                        "as this breaks the DICOM model of the real world.", false)
-      .SetRequestField("PrivateCreator", RestApiCallDocumentation::Type_String,
+      .SetRequestField(PRIVATE_CREATOR, RestApiCallDocumentation::Type_String,
                        "The private creator to be used for private tags in `Replace`", false);
+
+    // This was existing, but undocumented in Orthanc <= 1.9.6
+    DocumentKeepSource(call);
   }
 
 
@@ -87,22 +111,25 @@ namespace Orthanc
   {
     // Check out "DicomModification::ParseAnonymizationRequest()"
     call.GetDocumentation()
-      .SetRequestField("Force", RestApiCallDocumentation::Type_Boolean,
+      .SetRequestField(FORCE, RestApiCallDocumentation::Type_Boolean,
                        "Allow the modification of tags related to DICOM identifiers, at the risk of "
                        "breaking the DICOM model of the real world", false)
       .SetRequestField("DicomVersion", RestApiCallDocumentation::Type_String,
                        "Version of the DICOM standard to be used for anonymization. Check out "
                        "configuration option `DeidentifyLogsDicomVersion` for possible values.", false)
-      .SetRequestField("KeepPrivateTags", RestApiCallDocumentation::Type_Boolean,
+      .SetRequestField(KEEP_PRIVATE_TAGS, RestApiCallDocumentation::Type_Boolean,
                        "Keep the private tags from the DICOM instances (defaults to `false`)", false)
-      .SetRequestField("Replace", RestApiCallDocumentation::Type_JsonObject,
-                       "Associative array to change the value of some DICOM tags in the DICOM instances", false)
-      .SetRequestField("Remove", RestApiCallDocumentation::Type_JsonListOfStrings,
-                       "List of additional tags to be removed from the DICOM instances", false)
-      .SetRequestField("Keep", RestApiCallDocumentation::Type_JsonListOfStrings,
-                       "List of DICOM tags whose value must not be destroyed by the anonymization", false)
-      .SetRequestField("PrivateCreator", RestApiCallDocumentation::Type_String,
+      .SetRequestField(REPLACE, RestApiCallDocumentation::Type_JsonObject,
+                       "Associative array to change the value of some DICOM tags in the DICOM instances. " INFO_SUBSEQUENCES, false)
+      .SetRequestField(REMOVE, RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "List of additional tags to be removed from the DICOM instances. " INFO_SUBSEQUENCES, false)
+      .SetRequestField(KEEP, RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "List of DICOM tags whose value must not be destroyed by the anonymization. " INFO_SUBSEQUENCES, false)
+      .SetRequestField(PRIVATE_CREATOR, RestApiCallDocumentation::Type_String,
                        "The private creator to be used for private tags in `Replace`", false);
+
+    // This was existing, but undocumented in Orthanc <= 1.9.6
+    DocumentKeepSource(call);
   }
 
 
@@ -142,12 +169,12 @@ namespace Orthanc
     if (call.ParseJsonRequest(request) &&
         request.isObject())
     {
-      bool patientNameReplaced;
-      target.ParseAnonymizationRequest(patientNameReplaced, request);
+      bool patientNameOverridden;
+      target.ParseAnonymizationRequest(patientNameOverridden, request);
 
-      if (patientNameReplaced)
+      if (!patientNameOverridden)
       {
-        // Overwrite the random Patient's Name by one that is more
+        // Override the random Patient's Name by one that is more
         // user-friendly (provided none was specified by the user)
         target.Replace(DICOM_TAG_PATIENT_NAME, GeneratePatientName(OrthancRestApi::GetContext(call)), true);
       }
@@ -205,6 +232,27 @@ namespace Orthanc
   }
 
 
+  static ResourceType DetectModifyLevel(const DicomModification& modification)
+  {
+    if (modification.IsReplaced(DICOM_TAG_PATIENT_ID))
+    {
+      return ResourceType_Patient;
+    }
+    else if (modification.IsReplaced(DICOM_TAG_STUDY_INSTANCE_UID))
+    {
+      return ResourceType_Study;
+    }
+    else if (modification.IsReplaced(DICOM_TAG_SERIES_INSTANCE_UID))
+    {
+      return ResourceType_Series;
+    }
+    else
+    {
+      return ResourceType_Instance;
+    }
+  }
+
+
   static void ModifyInstance(RestApiPostCall& call)
   {
     if (call.IsDocumentation())
@@ -226,24 +274,8 @@ namespace Orthanc
     Json::Value request;
     ParseModifyRequest(request, modification, call);
 
-    if (modification.IsReplaced(DICOM_TAG_PATIENT_ID))
-    {
-      modification.SetLevel(ResourceType_Patient);
-    }
-    else if (modification.IsReplaced(DICOM_TAG_STUDY_INSTANCE_UID))
-    {
-      modification.SetLevel(ResourceType_Study);
-    }
-    else if (modification.IsReplaced(DICOM_TAG_SERIES_INSTANCE_UID))
-    {
-      modification.SetLevel(ResourceType_Series);
-    }
-    else
-    {
-      modification.SetLevel(ResourceType_Instance);
-    }
+    modification.SetLevel(DetectModifyLevel(modification));
 
-    static const char* TRANSCODE = "Transcode";
     if (request.isMember(TRANSCODE))
     {
       std::string s = SerializationToolbox::ReadString(request, TRANSCODE);
@@ -295,7 +327,6 @@ namespace Orthanc
   static void SetKeepSource(CleaningInstancesJob& job,
                             const Json::Value& body)
   {
-    static const char* KEEP_SOURCE = "KeepSource";
     if (body.isMember(KEEP_SOURCE))
     {
       job.SetKeepSource(SerializationToolbox::ReadBoolean(body, KEEP_SOURCE));
@@ -307,23 +338,37 @@ namespace Orthanc
                                     bool isAnonymization,
                                     RestApiPostCall& call,
                                     const Json::Value& body,
-                                    ResourceType level)
+                                    ResourceType outputLevel /* unused for multiple resources */,
+                                    bool isSingleResource,
+                                    const std::set<std::string>& resources)
   {
     ServerContext& context = OrthancRestApi::GetContext(call);
 
     std::unique_ptr<ResourceModificationJob> job(new ResourceModificationJob(context));
 
-    job->SetModification(modification.release(), level, isAnonymization);
+    if (isSingleResource)  // This notably configures the output format
+    {
+      job->SetSingleResourceModification(modification.release(), outputLevel, isAnonymization);
+    }
+    else
+    {
+      job->SetMultipleResourcesModification(modification.release(), isAnonymization);
+    }
+    
     job->SetOrigin(call);
     SetKeepSource(*job, body);
 
-    static const char* TRANSCODE = "Transcode";
     if (body.isMember(TRANSCODE))
     {
       job->SetTranscode(SerializationToolbox::ReadString(body, TRANSCODE));
     }
+
+    for (std::set<std::string>::const_iterator
+           it = resources.begin(); it != resources.end(); ++it)
+    {
+      context.AddChildInstances(*job, *it);
+    }
     
-    context.AddChildInstances(*job, call.GetUriComponent("id", ""));
     job->AddTrailingStep();
 
     OrthancRestApi::GetApi(call).SubmitCommandsJob
@@ -331,6 +376,35 @@ namespace Orthanc
   }
 
 
+  static void SubmitModificationJob(std::unique_ptr<DicomModification>& modification,
+                                    bool isAnonymization,
+                                    RestApiPostCall& call,
+                                    const Json::Value& body,
+                                    ResourceType outputLevel)
+  {
+    // This was the only flavor in Orthanc <= 1.9.3
+    std::set<std::string> resources;
+    resources.insert(call.GetUriComponent("id", ""));
+    
+    SubmitModificationJob(modification, isAnonymization, call, body, outputLevel,
+                          true /* single resource */, resources);
+  }
+
+  
+  static void SubmitBulkJob(std::unique_ptr<DicomModification>& modification,
+                            bool isAnonymization,
+                            RestApiPostCall& call,
+                            const Json::Value& body)
+  {
+    std::set<std::string> resources;
+    SerializationToolbox::ReadSetOfStrings(resources, body, RESOURCES);
+
+    SubmitModificationJob(modification, isAnonymization,
+                          call, body, ResourceType_Instance /* arbitrary value, unused */,
+                          false /* multiple resources */, resources);
+  }
+
+
   template <enum ResourceType resourceType>
   static void ModifyResource(RestApiPostCall& call)
   {
@@ -356,12 +430,54 @@ namespace Orthanc
     ParseModifyRequest(body, *modification, call);
 
     modification->SetLevel(resourceType);
-
+    
     SubmitModificationJob(modification, false /* not an anonymization */,
                           call, body, resourceType);
   }
 
 
+  // New in Orthanc 1.9.4
+  static void BulkModify(RestApiPostCall& call)
+  {
+    if (call.IsDocumentation())
+    {
+      OrthancRestApi::DocumentSubmitCommandsJob(call);
+      DocumentModifyOptions(call);
+      call.GetDocumentation()
+        .SetTag("System")
+        .SetSummary("Modify a set of resources")
+        .SetRequestField(RESOURCES, RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "List of the Orthanc identifiers of the patients/studies/series/instances of interest.", true)
+        .SetRequestField(LEVEL, RestApiCallDocumentation::Type_String,
+                         "Level of the modification (`Patient`, `Study`, `Series` or `Instance`). If absent, "
+                         "the level defaults to `Instance`, but is set to `Patient` if `PatientID` is modified, "
+                         "to `Study` if `StudyInstanceUID` is modified, or to `Series` if `SeriesInstancesUID` "
+                         "is modified. (new in Orthanc 1.9.7)", false)
+        .SetDescription("Start a job that will modify all the DICOM patients, studies, series or instances "
+                        "whose identifiers are provided in the `Resources` field.")
+        .AddAnswerType(MimeType_Json, "The list of all the resources that have been altered by this modification");
+      return;
+    }
+    
+    std::unique_ptr<DicomModification> modification(new DicomModification);
+
+    Json::Value body;
+    ParseModifyRequest(body, *modification, call);
+
+    if (body.isMember(LEVEL))
+    {
+      // This case was introduced in Orthanc 1.9.7
+      modification->SetLevel(StringToResourceType(body[LEVEL].asCString()));
+    }
+    else
+    {
+      modification->SetLevel(DetectModifyLevel(*modification));
+    }
+
+    SubmitBulkJob(modification, false /* not an anonymization */, call, body);
+  }
+
+
   template <enum ResourceType resourceType>
   static void AnonymizeResource(RestApiPostCall& call)
   {
@@ -391,6 +507,33 @@ namespace Orthanc
   }
 
 
+  // New in Orthanc 1.9.4
+  static void BulkAnonymize(RestApiPostCall& call)
+  {
+    if (call.IsDocumentation())
+    {
+      OrthancRestApi::DocumentSubmitCommandsJob(call);
+      DocumentAnonymizationOptions(call);
+      call.GetDocumentation()
+        .SetTag("System")
+        .SetSummary("Anonymize a set of resources")
+        .SetRequestField(RESOURCES, RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "List of the Orthanc identifiers of the patients/studies/series/instances of interest.", true)
+        .SetDescription("Start a job that will anonymize all the DICOM patients, studies, series or instances "
+                        "whose identifiers are provided in the `Resources` field.")
+        .AddAnswerType(MimeType_Json, "The list of all the resources that have been created by this anonymization");
+      return;
+    }
+
+    std::unique_ptr<DicomModification> modification(new DicomModification);
+
+    Json::Value body;
+    ParseAnonymizationRequest(body, *modification, call);
+
+    SubmitBulkJob(modification, true /* anonymization */, call, body);
+  }
+
+
   static void StoreCreatedInstance(std::string& id /* out */,
                                    RestApiPostCall& call,
                                    ParsedDicomFile& dicom,
@@ -400,16 +543,16 @@ namespace Orthanc
     toStore->SetOrigin(DicomInstanceOrigin::FromRest(call));
 
     ServerContext& context = OrthancRestApi::GetContext(call);
-    StoreStatus status = context.Store(id, *toStore, StoreInstanceMode_Default);
+    ServerContext::StoreResult result = context.Store(id, *toStore, StoreInstanceMode_Default);
 
-    if (status == StoreStatus_Failure)
+    if (result.GetStatus() == StoreStatus_Failure)
     {
       throw OrthancException(ErrorCode_CannotStoreInstance);
     }
 
     if (sendAnswer)
     {
-      OrthancRestApi::GetApi(call).AnswerStoredInstance(call, *toStore, status, id);
+      OrthancRestApi::GetApi(call).AnswerStoredInstance(call, *toStore, result.GetStatus(), id);
     }
   }
 
@@ -528,16 +671,16 @@ namespace Orthanc
         }
         else if (content[i].type() == Json::objectValue)
         {
-          if (!content[i].isMember("Content"))
+          if (!content[i].isMember(CONTENT))
           {
             throw OrthancException(ErrorCode_CreateDicomNoPayload);
           }
 
-          payload = &content[i]["Content"];
+          payload = &content[i][CONTENT];
 
-          if (content[i].isMember("Tags"))
+          if (content[i].isMember(TAGS))
           {
-            InjectTags(*dicom, content[i]["Tags"], decodeBinaryTags, privateCreator, force);
+            InjectTags(*dicom, content[i][TAGS], decodeBinaryTags, privateCreator, force);
           }
         }
 
@@ -579,13 +722,7 @@ namespace Orthanc
   static void CreateDicomV2(RestApiPostCall& call,
                             const Json::Value& request)
   {
-    static const char* const CONTENT = "Content";
-    static const char* const FORCE = "Force";
-    static const char* const INTERPRET_BINARY_TAGS = "InterpretBinaryTags";
-    static const char* const PARENT = "Parent";
-    static const char* const PRIVATE_CREATOR = "PrivateCreator";
     static const char* const SPECIFIC_CHARACTER_SET_2 = "SpecificCharacterSet";
-    static const char* const TAGS = "Tags";
     static const char* const TYPE = "Type";
     static const char* const VALUE = "Value";
     
@@ -659,7 +796,7 @@ namespace Orthanc
 
       // Choose the same encoding as the parent resource
       {
-        static const char* SPECIFIC_CHARACTER_SET = "0008,0005";
+        static const char* const SPECIFIC_CHARACTER_SET = "0008,0005";
 
         if (siblingTags.isMember(SPECIFIC_CHARACTER_SET))
         {
@@ -838,25 +975,25 @@ namespace Orthanc
         .SetTag("System")
         .SetSummary("Create one DICOM instance")
         .SetDescription("Create one DICOM instance, and store it into Orthanc")
-        .SetRequestField("Tags", RestApiCallDocumentation::Type_JsonObject,
+        .SetRequestField(TAGS, RestApiCallDocumentation::Type_JsonObject,
                          "Associative array containing the tags of the new instance to be created", true)
-        .SetRequestField("Content", RestApiCallDocumentation::Type_String,
+        .SetRequestField(CONTENT, RestApiCallDocumentation::Type_String,
                          "This field can be used to embed an image (pixel data) or a PDF inside the created DICOM instance. "
                          "The PNG image, the JPEG image or the PDF file must be provided using their "
                          "[data URI scheme encoding](https://en.wikipedia.org/wiki/Data_URI_scheme). "
                          "This field can possibly contain a JSON array, in which case a DICOM series is created "
                          "containing one DICOM instance for each item in the `Content` field.", false)
-        .SetRequestField("Parent", RestApiCallDocumentation::Type_String,
+        .SetRequestField(PARENT, RestApiCallDocumentation::Type_String,
                          "If present, the newly created instance will be attached to the parent DICOM resource "
                          "whose Orthanc identifier is contained in this field. The DICOM tags of the parent "
                          "modules in the DICOM hierarchy will be automatically copied to the newly created instance.", false)
-        .SetRequestField("InterpretBinaryTags", RestApiCallDocumentation::Type_Boolean,
+        .SetRequestField(INTERPRET_BINARY_TAGS, RestApiCallDocumentation::Type_Boolean,
                          "If some value in the `Tags` associative array is formatted according to some "
                          "[data URI scheme encoding](https://en.wikipedia.org/wiki/Data_URI_scheme), "
                          "whether this value is decoded to a binary value or kept as such (`true` by default)", false)
-        .SetRequestField("PrivateCreator", RestApiCallDocumentation::Type_String,
+        .SetRequestField(PRIVATE_CREATOR, RestApiCallDocumentation::Type_String,
                          "The private creator to be used for private tags in `Tags`", false)
-        .SetRequestField("Force", RestApiCallDocumentation::Type_Boolean,
+        .SetRequestField(FORCE, RestApiCallDocumentation::Type_Boolean,
                          "Avoid the consistency checks for the DICOM tags that enforce the DICOM model of the real-world. "
                          "You can notably use this flag if you need to manually set the tags `StudyInstanceUID`, "
                          "`SeriesInstanceUID`, or `SOPInstanceUID`. Be careful with this feature.", false)
@@ -872,7 +1009,7 @@ namespace Orthanc
       throw OrthancException(ErrorCode_BadRequest);
     }
 
-    if (request.isMember("Tags"))
+    if (request.isMember(TAGS))
     {
       CreateDicomV2(call, request);
     }
@@ -897,22 +1034,25 @@ namespace Orthanc
         .SetTag("Studies")
         .SetSummary("Split study")
         .SetDescription("Start a new job so as to split the DICOM study whose Orthanc identifier is provided in the URL, "
-                        "by taking some of its children series out of it and putting them into a brand new study (this "
-                        "new study is created by setting the `StudyInstanceUID` tag to a random identifier): "
+                        "by taking some of its children series or instances out of it and putting them into a brand new study "
+                        "(this new study is created by setting the `StudyInstanceUID` tag to a random identifier): "
                         "https://book.orthanc-server.com/users/anonymization.html#splitting")
         .SetUriArgument("id", "Orthanc identifier of the study of interest")
-        .SetRequestField("Series", RestApiCallDocumentation::Type_JsonListOfStrings,
-                         "The list of series to be separated from the parent study (mandatory option). "
-                         "These series must all be children of the same source study, that is specified in the URI.", true)
-        .SetRequestField("Replace", RestApiCallDocumentation::Type_JsonObject,
+        .SetRequestField(SERIES, RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "The list of series to be separated from the parent study. "
+                         "These series must all be children of the same source study, that is specified in the URI.", false)
+        .SetRequestField(REPLACE, RestApiCallDocumentation::Type_JsonObject,
                          "Associative array to change the value of some DICOM tags in the new study. "
                          "These tags must be part of the \"Patient Module Attributes\" or the \"General Study "
                          "Module Attributes\", as specified by the DICOM 2011 standard in Tables C.7-1 and C.7-3.", false)
-        .SetRequestField("Remove", RestApiCallDocumentation::Type_JsonListOfStrings,
+        .SetRequestField(REMOVE, RestApiCallDocumentation::Type_JsonListOfStrings,
                          "List of tags that must be removed in the new study (from the same modules as in the `Replace` option)", false)
-        .SetRequestField("KeepSource", RestApiCallDocumentation::Type_Boolean,
-                         "If set to `true`, instructs Orthanc to keep a copy of the original series in the source study. "
-                         "By default, the original series are deleted from Orthanc.", false);
+        .SetRequestField(KEEP_SOURCE, RestApiCallDocumentation::Type_Boolean,
+                         "If set to `true`, instructs Orthanc to keep a copy of the original series/instances in the source study. "
+                         "By default, the original series/instances are deleted from Orthanc.", false)
+        .SetRequestField(INSTANCES, RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "The list of instances to be separated from the parent study. "
+                         "These instances must all be children of the same source study, that is specified in the URI.", false);
       return;
     }
     
@@ -930,19 +1070,40 @@ namespace Orthanc
     std::unique_ptr<SplitStudyJob> job(new SplitStudyJob(context, study));    
     job->SetOrigin(call);
 
-    std::vector<std::string> series;
-    SerializationToolbox::ReadArrayOfStrings(series, request, "Series");
-
-    for (size_t i = 0; i < series.size(); i++)
+    bool ok = false;
+    if (request.isMember(SERIES))
     {
-      job->AddSourceSeries(series[i]);
+      std::vector<std::string> series;
+      SerializationToolbox::ReadArrayOfStrings(series, request, SERIES);
+
+      for (size_t i = 0; i < series.size(); i++)
+      {
+        job->AddSourceSeries(series[i]);
+        ok = true;
+      }
     }
+
+    if (request.isMember(INSTANCES))
+    {
+      std::vector<std::string> instances;
+      SerializationToolbox::ReadArrayOfStrings(instances, request, INSTANCES);
+
+      for (size_t i = 0; i < instances.size(); i++)
+      {
+        job->AddSourceInstance(instances[i]);
+        ok = true;
+      }
+    }
+
+    if (!ok)
+    {
+      throw OrthancException(ErrorCode_BadRequest, "Both the \"Series\" and the \"Instances\" fields are missing");
+    }    
     
     job->AddTrailingStep();
 
     SetKeepSource(*job, request);
 
-    static const char* REMOVE = "Remove";
     if (request.isMember(REMOVE))
     {
       if (request[REMOVE].type() != Json::arrayValue)
@@ -963,7 +1124,6 @@ namespace Orthanc
       }
     }
 
-    static const char* REPLACE = "Replace";
     if (request.isMember(REPLACE))
     {
       if (request[REPLACE].type() != Json::objectValue)
@@ -1001,13 +1161,13 @@ namespace Orthanc
       call.GetDocumentation()
         .SetTag("Studies")
         .SetSummary("Merge study")
-        .SetDescription("Start a new job so as to move some DICOM series into the DICOM study whose Orthanc identifier "
+        .SetDescription("Start a new job so as to move some DICOM resources into the DICOM study whose Orthanc identifier "
                         "is provided in the URL: https://book.orthanc-server.com/users/anonymization.html#merging")
         .SetUriArgument("id", "Orthanc identifier of the study of interest")
-        .SetRequestField("Resources", RestApiCallDocumentation::Type_JsonListOfStrings,
-                         "The list of DICOM resources (patients, studies, series, and/or instances) to be merged "
+        .SetRequestField(RESOURCES, RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "The list of DICOM resources (studies, series, and/or instances) to be merged "
                          "into the study of interest (mandatory option)", true)
-        .SetRequestField("KeepSource", RestApiCallDocumentation::Type_Boolean,
+        .SetRequestField(KEEP_SOURCE, RestApiCallDocumentation::Type_Boolean,
                          "If set to `true`, instructs Orthanc to keep a copy of the original resources in their source study. "
                          "By default, the original resources are deleted from Orthanc.", false);
       return;
@@ -1028,7 +1188,7 @@ namespace Orthanc
     job->SetOrigin(call);
 
     std::vector<std::string> resources;
-    SerializationToolbox::ReadArrayOfStrings(resources, request, "Resources");
+    SerializationToolbox::ReadArrayOfStrings(resources, request, RESOURCES);
 
     for (size_t i = 0; i < resources.size(); i++)
     {
@@ -1050,11 +1210,13 @@ namespace Orthanc
     Register("/series/{id}/modify", ModifyResource<ResourceType_Series>);
     Register("/studies/{id}/modify", ModifyResource<ResourceType_Study>);
     Register("/patients/{id}/modify", ModifyResource<ResourceType_Patient>);
+    Register("/tools/bulk-modify", BulkModify);
 
     Register("/instances/{id}/anonymize", AnonymizeInstance);
     Register("/series/{id}/anonymize", AnonymizeResource<ResourceType_Series>);
     Register("/studies/{id}/anonymize", AnonymizeResource<ResourceType_Study>);
     Register("/patients/{id}/anonymize", AnonymizeResource<ResourceType_Patient>);
+    Register("/tools/bulk-anonymize", BulkAnonymize);
 
     Register("/tools/create-dicom", CreateDicom);
 
