@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,6 +28,7 @@
 #include "../../../OrthancFramework/Sources/Logging.h"
 #include "../../../OrthancFramework/Sources/MetricsRegistry.h"
 #include "../../../OrthancFramework/Sources/SerializationToolbox.h"
+#include "../OrthancConfiguration.h"
 #include "../ServerContext.h"
 
 #include <boost/algorithm/string/predicate.hpp>
@@ -63,15 +53,19 @@ namespace Orthanc
 
 
   static void SetupResourceAnswer(Json::Value& result,
-                                  DicomInstanceToStore& instance,
+                                  const DicomInstanceToStore& instance,
                                   StoreStatus status,
                                   const std::string& instanceId)
   {
     SetupResourceAnswer(result, instanceId, ResourceType_Instance, status);
 
-    result["ParentPatient"] = instance.GetHasher().HashPatient();
-    result["ParentStudy"] = instance.GetHasher().HashStudy();
-    result["ParentSeries"] = instance.GetHasher().HashSeries();
+    DicomMap summary;
+    instance.GetSummary(summary);
+
+    DicomInstanceHasher hasher(summary);
+    result["ParentPatient"] = hasher.HashPatient();
+    result["ParentStudy"] = hasher.HashStudy();
+    result["ParentSeries"] = hasher.HashSeries();
   }
 
 
@@ -186,19 +180,18 @@ namespace Orthanc
         if (!content.empty())
         {
           LOG(INFO) << "Uploading DICOM file from ZIP archive: " << filename;
-          
-          DicomInstanceToStore toStore;
-          toStore.SetOrigin(DicomInstanceOrigin::FromRest(call));
-          toStore.SetBuffer(content.c_str(), content.size());
+
+          std::unique_ptr<DicomInstanceToStore> toStore(DicomInstanceToStore::CreateFromBuffer(content));
+          toStore->SetOrigin(DicomInstanceOrigin::FromRest(call));
 
           std::string publicId;
 
           try
           {
-            StoreStatus status = context.Store(publicId, toStore, StoreInstanceMode_Default);
+            ServerContext::StoreResult result = context.Store(publicId, *toStore, StoreInstanceMode_Default);
 
             Json::Value info;
-            SetupResourceAnswer(info, toStore, status, publicId);
+            SetupResourceAnswer(info, *toStore, result.GetStatus(), publicId);
             answer.append(info);
           }
           catch (OrthancException& e)
@@ -207,6 +200,15 @@ namespace Orthanc
             {
               LOG(ERROR) << "Cannot import non-DICOM file from ZIP archive: " << filename;
             }
+            else if (e.GetErrorCode() == ErrorCode_InexistentTag)
+            {
+              /**
+               * Allow upload of ZIP archives containing a DICOMDIR
+               * file (new in Orthanc 1.9.7):
+               * https://groups.google.com/g/orthanc-users/c/sgBU89o4nhU/m/kbRAYiQUAAAJ
+               **/
+              LOG(ERROR) << "Ignoring what is probably a DICOMDIR file within a ZIP archive: \"" << filename << "\"";
+            }
             else
             {
               throw;
@@ -223,24 +225,25 @@ namespace Orthanc
       // latter can possibly store a reference to the former (*)
       std::string dicom;
 
-      DicomInstanceToStore toStore;
-      toStore.SetOrigin(DicomInstanceOrigin::FromRest(call));
+      std::unique_ptr<DicomInstanceToStore> toStore;
 
       if (boost::iequals(call.GetHttpHeader("content-encoding", ""), "gzip"))
       {
         GzipCompressor compressor;
         compressor.Uncompress(dicom, call.GetBodyData(), call.GetBodySize());
-        toStore.SetBuffer(dicom.c_str(), dicom.size());  // (*)
+        toStore.reset(DicomInstanceToStore::CreateFromBuffer(dicom));  // (*)
       }
       else
       {
-        toStore.SetBuffer(call.GetBodyData(), call.GetBodySize());
+        toStore.reset(DicomInstanceToStore::CreateFromBuffer(call.GetBodyData(), call.GetBodySize()));
       }    
 
-      std::string publicId;
-      StoreStatus status = context.Store(publicId, toStore, StoreInstanceMode_Default);
+      toStore->SetOrigin(DicomInstanceOrigin::FromRest(call));
 
-      OrthancRestApi::GetApi(call).AnswerStoredInstance(call, toStore, status, publicId);
+      std::string publicId;
+      ServerContext::StoreResult result = context.Store(publicId, *toStore, StoreInstanceMode_Default);
+
+      OrthancRestApi::GetApi(call).AnswerStoredInstance(call, *toStore, result.GetStatus(), publicId);
     }
   }
 
@@ -455,4 +458,116 @@ namespace Orthanc
       .SetRequestField(KEY_PERMISSIVE, RestApiCallDocumentation::Type_Boolean,
                        "If `true`, ignore errors during the individual steps of the job.", false);
   }
+
+
+  static const std::string GET_SIMPLIFY = "simplify";
+  static const std::string GET_FULL = "full";
+  static const std::string GET_SHORT = "short";
+
+  static const std::string POST_SIMPLIFY = "Simplify";
+  static const std::string POST_FULL = "Full";
+  static const std::string POST_SHORT = "Short";
+
+  static const std::string DOCUMENT_SIMPLIFY =
+    "report the DICOM tags in human-readable format (using the symbolic name of the tags)";
+
+  static const std::string DOCUMENT_SHORT =
+    "report the DICOM tags in hexadecimal format";
+
+  static const std::string DOCUMENT_FULL =
+    "report the DICOM tags in full format (tags indexed by their hexadecimal "
+    "format, associated with their symbolic name and their value)";
+
+
+  DicomToJsonFormat OrthancRestApi::GetDicomFormat(const RestApiGetCall& call,
+                                                   DicomToJsonFormat defaultFormat)
+  {
+    if (call.HasArgument(GET_SIMPLIFY))
+    {
+      return DicomToJsonFormat_Human;
+    }
+    else if (call.HasArgument(GET_SHORT))
+    {
+      return DicomToJsonFormat_Short;
+    }
+    else if (call.HasArgument(GET_FULL))
+    {
+      return DicomToJsonFormat_Full;
+    }
+    else
+    {
+      return defaultFormat;
+    }
+  }
+
+
+  DicomToJsonFormat OrthancRestApi::GetDicomFormat(const Json::Value& body,
+                                                   DicomToJsonFormat defaultFormat)
+  {
+    if (body.isMember(POST_SIMPLIFY) &&
+        SerializationToolbox::ReadBoolean(body, POST_SIMPLIFY))
+    {
+      return DicomToJsonFormat_Human;
+    }
+    else if (body.isMember(POST_SHORT) &&
+             SerializationToolbox::ReadBoolean(body, POST_SHORT))
+    {
+      return DicomToJsonFormat_Short;
+    }
+    else if (body.isMember(POST_FULL) &&
+             SerializationToolbox::ReadBoolean(body, POST_FULL))
+    {
+      return DicomToJsonFormat_Full;
+    }
+    else
+    {
+      return defaultFormat;
+    }
+  }
+
+
+  void OrthancRestApi::DocumentDicomFormat(RestApiGetCall& call,
+                                           DicomToJsonFormat defaultFormat)
+  {
+    if (defaultFormat != DicomToJsonFormat_Human)
+    {
+      call.GetDocumentation().SetHttpGetArgument(
+        GET_SIMPLIFY, RestApiCallDocumentation::Type_Boolean, "If present, " + DOCUMENT_SIMPLIFY, false);
+    }
+    
+    if (defaultFormat != DicomToJsonFormat_Short)
+    {
+      call.GetDocumentation().SetHttpGetArgument(
+        GET_SHORT, RestApiCallDocumentation::Type_Boolean, "If present, " + DOCUMENT_SHORT, false);
+    }
+    
+    if (defaultFormat != DicomToJsonFormat_Full)
+    {
+      call.GetDocumentation().SetHttpGetArgument(
+        GET_FULL, RestApiCallDocumentation::Type_Boolean, "If present, " + DOCUMENT_FULL, false);
+    }    
+  }
+  
+  
+  void OrthancRestApi::DocumentDicomFormat(RestApiPostCall& call,
+                                           DicomToJsonFormat defaultFormat)
+  {
+    if (defaultFormat != DicomToJsonFormat_Human)
+    {
+      call.GetDocumentation().SetRequestField(POST_SIMPLIFY, RestApiCallDocumentation::Type_Boolean,
+                                              "If set to `true`, " + DOCUMENT_SIMPLIFY, false);
+    }
+    
+    if (defaultFormat != DicomToJsonFormat_Short)
+    {
+      call.GetDocumentation().SetRequestField(POST_SHORT, RestApiCallDocumentation::Type_Boolean,
+                                              "If set to `true`, " + DOCUMENT_SHORT, false);
+    }
+    
+    if (defaultFormat != DicomToJsonFormat_Full)
+    {
+      call.GetDocumentation().SetRequestField(POST_FULL, RestApiCallDocumentation::Type_Boolean,
+                                              "If set to `true`, " + DOCUMENT_FULL, false);
+    }
+  }
 }
