@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,7 +47,7 @@ namespace Orthanc
     bool                            resetRequestReceived_;
     MetricsRegistry::SharedMetrics  activeRequests_;
 
-    void RegisterSystem();
+    void RegisterSystem(bool orthancExplorerEnabled);
 
     void RegisterChanges();
 
@@ -75,7 +64,8 @@ namespace Orthanc
     static void ShutdownOrthanc(RestApiPostCall& call);
 
   public:
-    OrthancRestApi(ServerContext& context);
+    explicit OrthancRestApi(ServerContext& context,
+                            bool orthancExplorerEnabled);
 
     virtual bool Handle(HttpOutput& output,
                         RequestOrigin origin,
@@ -83,8 +73,8 @@ namespace Orthanc
                         const char* username,
                         HttpMethod method,
                         const UriComponents& uri,
-                        const Arguments& headers,
-                        const GetArguments& getArguments,
+                        const HttpToolbox::Arguments& headers,
+                        const HttpToolbox::GetArguments& getArguments,
                         const void* bodyData,
                         size_t bodySize) ORTHANC_OVERRIDE;
 
@@ -139,5 +129,21 @@ namespace Orthanc
                            SetOfCommandsJob* job,
                            bool isDefaultSynchronous,
                            const Json::Value& body) const;
+
+    static void DocumentSubmitGenericJob(RestApiPostCall& call);
+
+    static void DocumentSubmitCommandsJob(RestApiPostCall& call);
+
+    static DicomToJsonFormat GetDicomFormat(const RestApiGetCall& call,
+                                            DicomToJsonFormat defaultFormat);
+
+    static DicomToJsonFormat GetDicomFormat(const Json::Value& body,
+                                            DicomToJsonFormat defaultFormat);
+
+    static void DocumentDicomFormat(RestApiGetCall& call,
+                                    DicomToJsonFormat defaultFormat);
+
+    static void DocumentDicomFormat(RestApiPostCall& call,
+                                    DicomToJsonFormat defaultFormat);
   };
 }
