@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,7 +33,16 @@
 
 namespace Orthanc
 {
-  class ResourceModificationJob::Output : public boost::noncopyable
+  static void FormatResource(Json::Value& target,
+                             ResourceType level,
+                             const std::string& id)
+  {
+    target["Type"] = EnumerationToString(level);
+    target["ID"] = id;
+    target["Path"] = GetBasePath(level, id);
+  }
+  
+  class ResourceModificationJob::SingleOutput : public IOutput
   {
   private:
     ResourceType  level_;
@@ -53,7 +51,7 @@ namespace Orthanc
     std::string   patientId_;
 
   public:
-    explicit Output(ResourceType level) :
+    explicit SingleOutput(ResourceType level) :
       level_(level),
       isFirst_(true)
     {
@@ -65,13 +63,7 @@ namespace Orthanc
       }            
     }
 
-    ResourceType GetLevel() const
-    {
-      return level_;
-    }
-    
-
-    void Update(DicomInstanceHasher& hasher)
+    virtual void Update(DicomInstanceHasher& hasher) ORTHANC_OVERRIDE
     {
       if (isFirst_)
       {
@@ -98,40 +90,78 @@ namespace Orthanc
       }
     }
 
-
-    bool Format(Json::Value& target)
+    virtual void Format(Json::Value& target) const ORTHANC_OVERRIDE
     {
-      if (isFirst_)
+      assert(target.type() == Json::objectValue);
+
+      if (!isFirst_)
       {
-        return false;
-      }
-      else
-      {
-        target = Json::objectValue;
-        target["Type"] = EnumerationToString(level_);
-        target["ID"] = id_;
-        target["Path"] = GetBasePath(level_, id_);
+        FormatResource(target, level_, id_);
         target["PatientID"] = patientId_;
-        return true;
       }
     }
 
-  
-    bool GetIdentifier(std::string& id)
+    virtual bool IsSingleResource() const ORTHANC_OVERRIDE
     {
-      if (isFirst_)
-      {
-        return false;
-      }
-      else
-      {
-        id = id_;
-        return true;
-      }
+      return true;
+    }
+
+    ResourceType GetLevel() const
+    {
+      return level_;
     }
   };
     
 
+  class ResourceModificationJob::MultipleOutputs : public IOutput
+  {
+  private:
+    static void FormatResources(Json::Value& target,
+                                ResourceType level,
+                                const std::set<std::string>& resources)
+    {
+      assert(target.type() == Json::arrayValue);
+
+      for (std::set<std::string>::const_iterator
+             it = resources.begin(); it != resources.end(); ++it)
+      {
+        Json::Value item = Json::objectValue;
+        FormatResource(item, level, *it);
+        target.append(item);        
+      }
+    }
+    
+    std::set<std::string>  instances_;
+    std::set<std::string>  series_;
+    std::set<std::string>  studies_;
+    std::set<std::string>  patients_;
+
+  public:
+    virtual void Update(DicomInstanceHasher& hasher) ORTHANC_OVERRIDE
+    {
+      instances_.insert(hasher.HashInstance());
+      series_.insert(hasher.HashSeries());
+      studies_.insert(hasher.HashStudy());
+      patients_.insert(hasher.HashPatient());
+    }
+
+    virtual void Format(Json::Value& target) const ORTHANC_OVERRIDE
+    {
+      assert(target.type() == Json::objectValue);
+      Json::Value resources = Json::arrayValue;
+      FormatResources(resources, ResourceType_Instance, instances_);
+      FormatResources(resources, ResourceType_Series, series_);
+      FormatResources(resources, ResourceType_Study, studies_);
+      FormatResources(resources, ResourceType_Patient, patients_);
+      target["Resources"] = resources;
+    }
+
+    virtual bool IsSingleResource() const ORTHANC_OVERRIDE
+    {
+      return false;
+    }
+  };
+    
 
 
   bool ResourceModificationJob::HandleInstance(const std::string& instance)
@@ -210,9 +240,8 @@ namespace Orthanc
 
     assert(modifiedUid == IDicomTranscoder::GetSopInstanceUid(modified->GetDcmtkObject()));
 
-    DicomInstanceToStore toStore;
-    toStore.SetOrigin(origin_);
-    toStore.SetParsedDicomFile(*modified);
+    std::unique_ptr<DicomInstanceToStore> toStore(DicomInstanceToStore::CreateFromParsedDicomFile(*modified));
+    toStore->SetOrigin(origin_);
 
 
     /**
@@ -228,21 +257,21 @@ namespace Orthanc
 
     if (originalHasher->HashSeries() != modifiedHasher.HashSeries())
     {
-      toStore.AddMetadata(ResourceType_Series, metadataType, originalHasher->HashSeries());
+      toStore->AddMetadata(ResourceType_Series, metadataType, originalHasher->HashSeries());
     }
 
     if (originalHasher->HashStudy() != modifiedHasher.HashStudy())
     {
-      toStore.AddMetadata(ResourceType_Study, metadataType, originalHasher->HashStudy());
+      toStore->AddMetadata(ResourceType_Study, metadataType, originalHasher->HashStudy());
     }
 
     if (originalHasher->HashPatient() != modifiedHasher.HashPatient())
     {
-      toStore.AddMetadata(ResourceType_Patient, metadataType, originalHasher->HashPatient());
+      toStore->AddMetadata(ResourceType_Patient, metadataType, originalHasher->HashPatient());
     }
 
     assert(instance == originalHasher->HashInstance());
-    toStore.AddMetadata(ResourceType_Instance, metadataType, instance);
+    toStore->AddMetadata(ResourceType_Instance, metadataType, instance);
 
 
     /**
@@ -250,8 +279,8 @@ namespace Orthanc
      **/
 
     std::string modifiedInstance;
-    if (GetContext().Store(modifiedInstance, toStore,
-                           StoreInstanceMode_Default) != StoreStatus_Success)
+    ServerContext::StoreResult result = GetContext().Store(modifiedInstance, *toStore, StoreInstanceMode_Default);
+    if (result.GetStatus() != StoreStatus_Success)
     {
       throw OrthancException(ErrorCode_CannotStoreInstance,
                              "Error while storing a modified instance " + instance);
@@ -272,7 +301,6 @@ namespace Orthanc
 
   ResourceModificationJob::ResourceModificationJob(ServerContext& context) :
     CleaningInstancesJob(context, true /* by default, keep source */),
-    modification_(new DicomModification),
     isAnonymization_(false),
     transcode_(false),
     transferSyntax_(DicomTransferSyntax_LittleEndianExplicit)  // dummy initialization
@@ -280,9 +308,9 @@ namespace Orthanc
   }
 
 
-  void ResourceModificationJob::SetModification(DicomModification* modification,
-                                                ResourceType level,
-                                                bool isAnonymization)
+  void ResourceModificationJob::SetSingleResourceModification(DicomModification* modification,
+                                                              ResourceType outputLevel,
+                                                              bool isAnonymization)
   {
     if (modification == NULL)
     {
@@ -295,7 +323,27 @@ namespace Orthanc
     else
     {
       modification_.reset(modification);
-      output_.reset(new Output(level));
+      output_.reset(new SingleOutput(outputLevel));
+      isAnonymization_ = isAnonymization;
+    }
+  }
+
+
+  void ResourceModificationJob::SetMultipleResourcesModification(DicomModification* modification,
+                                                                 bool isAnonymization)
+  {
+    if (modification == NULL)
+    {
+      throw OrthancException(ErrorCode_NullPointer);
+    }
+    else if (IsStarted())
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else
+    {
+      modification_.reset(modification);
+      output_.reset(new MultipleOutputs);
       isAnonymization_ = isAnonymization;
     }
   }
@@ -388,6 +436,37 @@ namespace Orthanc
   }
 
 
+  bool ResourceModificationJob::IsSingleResourceModification() const
+  {
+    if (modification_.get() == NULL)
+    {
+      assert(output_.get() == NULL);
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else
+    {
+      assert(output_.get() != NULL);
+      return output_->IsSingleResource();
+    }
+  }
+  
+
+  ResourceType ResourceModificationJob::GetOutputLevel() const
+  {
+    if (IsSingleResourceModification())
+    {
+      assert(modification_.get() != NULL &&
+             output_.get() != NULL);
+      return dynamic_cast<const SingleOutput&>(*output_).GetLevel();
+    }
+    else
+    {
+      // Not applicable if multiple resources
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+  }
+
+
   void ResourceModificationJob::GetPublicContent(Json::Value& value)
   {
     CleaningInstancesJob::GetPublicContent(value);
@@ -410,6 +489,8 @@ namespace Orthanc
   static const char* ORIGIN = "Origin";
   static const char* IS_ANONYMIZATION = "IsAnonymization";
   static const char* TRANSCODE = "Transcode";
+  static const char* OUTPUT_LEVEL = "OutputLevel";
+  static const char* IS_SINGLE_RESOURCE = "IsSingleResource";
   
 
   ResourceModificationJob::ResourceModificationJob(ServerContext& context,
@@ -419,9 +500,7 @@ namespace Orthanc
   {
     assert(serialized.type() == Json::objectValue);
 
-    isAnonymization_ = SerializationToolbox::ReadBoolean(serialized, IS_ANONYMIZATION);
     origin_ = DicomInstanceOrigin(serialized[ORIGIN]);
-    modification_.reset(new DicomModification(serialized[MODIFICATION]));
 
     if (serialized.isMember(TRANSCODE))
     {
@@ -431,11 +510,62 @@ namespace Orthanc
     {
       transcode_ = false;
     }
+
+    bool isSingleResource;
+    if (serialized.isMember(IS_SINGLE_RESOURCE))
+    {
+      isSingleResource = SerializationToolbox::ReadBoolean(serialized, IS_SINGLE_RESOURCE);
+    }
+    else
+    {
+      isSingleResource = true;  // Backward compatibility with Orthanc <= 1.9.3
+    }
+
+    bool isAnonymization = SerializationToolbox::ReadBoolean(serialized, IS_ANONYMIZATION);
+    std::unique_ptr<DicomModification> modification(new DicomModification(serialized[MODIFICATION]));
+
+    if (isSingleResource)
+    {
+      ResourceType outputLevel;
+      
+      if (serialized.isMember(OUTPUT_LEVEL))
+      {
+        // New in Orthanc 1.9.4. This fixes an *incorrect* behavior in
+        // Orthanc <= 1.9.3, in which "outputLevel" would be set to
+        // "modification->GetLevel()"
+        outputLevel = StringToResourceType(SerializationToolbox::ReadString(serialized, OUTPUT_LEVEL).c_str());
+      }
+      else
+      {
+        // Use the buggy convention from Orthanc <= 1.9.3 (which is
+        // the only thing we have at hand)
+        outputLevel = modification->GetLevel();
+
+        if (outputLevel == ResourceType_Instance)
+        {
+          // This should never happen, but as "SingleOutput" doesn't
+          // support instance-level anonymization, don't take any risk
+          // and choose an arbitrary output level
+          outputLevel = ResourceType_Patient;
+        }
+      }
+      
+      SetSingleResourceModification(modification.release(), outputLevel, isAnonymization);
+    }
+    else
+    {
+      // New in Orthanc 1.9.4
+      SetMultipleResourcesModification(modification.release(), isAnonymization);
+    }
   }
   
   bool ResourceModificationJob::Serialize(Json::Value& value)
   {
-    if (!CleaningInstancesJob::Serialize(value))
+    if (modification_.get() == NULL)
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else if (!CleaningInstancesJob::Serialize(value))
     {
       return false;
     }
@@ -456,6 +586,13 @@ namespace Orthanc
       modification_->Serialize(tmp);
       value[MODIFICATION] = tmp;
 
+      // New in Orthanc 1.9.4
+      value[IS_SINGLE_RESOURCE] = IsSingleResourceModification();
+      if (IsSingleResourceModification())
+      {
+        value[OUTPUT_LEVEL] = EnumerationToString(GetOutputLevel());
+      }
+      
       return true;
     }
   }
