@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,10 +33,25 @@ namespace Orthanc
   class ResourceModificationJob : public CleaningInstancesJob
   {
   private:
-    class Output;
+    class IOutput : public boost::noncopyable
+    {
+    public:
+      virtual ~IOutput()
+      {
+      }
+
+      virtual void Update(DicomInstanceHasher& hasher) = 0;
+
+      virtual void Format(Json::Value& target) const = 0;
+
+      virtual bool IsSingleResource() const = 0;
+    };
+    
+    class SingleOutput;
+    class MultipleOutputs;
     
     std::unique_ptr<DicomModification>  modification_;
-    boost::shared_ptr<Output>           output_;
+    boost::shared_ptr<IOutput>          output_;
     bool                                isAnonymization_;
     DicomInstanceOrigin                 origin_;
     bool                                transcode_;
@@ -62,9 +66,14 @@ namespace Orthanc
     ResourceModificationJob(ServerContext& context,
                             const Json::Value& serialized);
 
-    void SetModification(DicomModification* modification,   // Takes ownership
-                         ResourceType level,
-                         bool isAnonymization);
+    // NB: The "outputLevel" only controls the output format, and
+    // might *not* be the same as "modification->GetLevel()"
+    void SetSingleResourceModification(DicomModification* modification,   // Takes ownership
+                                       ResourceType outputLevel,
+                                       bool isAnonymization);
+
+    void SetMultipleResourcesModification(DicomModification* modification,   // Takes ownership
+                                          bool isAnonymization);
 
     void SetOrigin(const DicomInstanceOrigin& origin);
 
@@ -95,6 +104,11 @@ namespace Orthanc
 
     void ClearTranscode();
 
+    bool IsSingleResourceModification() const;
+
+    // Only possible if "IsSingleResourceModification()"
+    ResourceType GetOutputLevel() const;
+
     virtual void Stop(JobStopReason reason) ORTHANC_OVERRIDE
     {
     }
