@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2022 Osimis S.A., Belgium
+ * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -140,6 +129,7 @@ namespace
 
     virtual bool GetOutput(std::string& output,
                            MimeType& mime,
+                           std::string& filename,
                            const std::string& key) ORTHANC_OVERRIDE
     {
       return false;
@@ -535,10 +525,10 @@ namespace
       dicom.Replace(DICOM_TAG_PATIENT_NAME, std::string("JODOGNE"),
                     false, DicomReplaceMode_InsertIfAbsent, "");
 
-      DicomInstanceToStore toStore;
-      toStore.SetParsedDicomFile(dicom);
+      std::unique_ptr<DicomInstanceToStore> toStore(DicomInstanceToStore::CreateFromParsedDicomFile(dicom));
 
-      return (context_->Store(id, toStore, StoreInstanceMode_Default) == StoreStatus_Success);
+      ServerContext::StoreResult result = context_->Store(id, *toStore, StoreInstanceMode_Default);
+      return (result.GetStatus() == StoreStatus_Success);
     }
   };
 }
@@ -699,6 +689,24 @@ TEST_F(OrthancJobsSerialization, Operations)
     ASSERT_EQ(RequestOrigin_Lua, tmp.GetRequestOrigin());
     ASSERT_TRUE(tmp.GetModification().IsRemoved(DICOM_TAG_STUDY_DESCRIPTION));
   }
+
+  {
+    std::unique_ptr<DicomModification> modification(new DicomModification);
+    modification->SetupAnonymization(DicomVersion_2021b);
+    
+    ModifyInstanceOperation operation(GetContext(), RequestOrigin_Lua, modification.release());
+
+    ASSERT_TRUE(CheckIdempotentSerialization(unserializer, operation));
+    operation.Serialize(s);
+  }
+
+  {
+    operation.reset(unserializer.UnserializeOperation(s));
+
+    const ModifyInstanceOperation& tmp = dynamic_cast<ModifyInstanceOperation&>(*operation);
+    ASSERT_EQ(RequestOrigin_Lua, tmp.GetRequestOrigin());
+    ASSERT_TRUE(tmp.GetModification().IsRemoved(DICOM_TAG_STUDY_DESCRIPTION));
+  }
 }
 
 
@@ -802,11 +810,16 @@ TEST_F(OrthancJobsSerialization, Jobs)
 
   {
     std::unique_ptr<DicomModification> modification(new DicomModification);
-    modification->SetupAnonymization(DicomVersion_2008);    
+    modification->SetupAnonymization(DicomVersion_2008);
+    modification->SetLevel(ResourceType_Series);
 
     ResourceModificationJob job(GetContext());
-    job.SetModification(modification.release(), ResourceType_Patient, true);
+    ASSERT_THROW(job.IsSingleResourceModification(), OrthancException);
+    job.SetSingleResourceModification(modification.release(), ResourceType_Patient, true);
     job.SetOrigin(DicomInstanceOrigin::FromLua());
+    ASSERT_TRUE(job.IsAnonymization());
+    ASSERT_TRUE(job.IsSingleResourceModification());
+    ASSERT_EQ(ResourceType_Patient, job.GetOutputLevel());
 
     job.AddTrailingStep();  // Necessary since 1.7.0
     ASSERT_TRUE(CheckIdempotentSetOfInstances(unserializer, job));
@@ -823,12 +836,32 @@ TEST_F(OrthancJobsSerialization, Jobs)
     ASSERT_THROW(tmp.GetTransferSyntax(), OrthancException);
     ASSERT_EQ(RequestOrigin_Lua, tmp.GetOrigin().GetRequestOrigin());
     ASSERT_TRUE(tmp.GetModification().IsRemoved(DICOM_TAG_STUDY_DESCRIPTION));
+    ASSERT_TRUE(tmp.IsSingleResourceModification());
+    ASSERT_EQ(ResourceType_Patient, tmp.GetOutputLevel());
+    ASSERT_EQ(ResourceType_Series, tmp.GetModification().GetLevel());
+  }
+
+  {
+    // Backward compatibility with Orthanc 1.9.3
+    ASSERT_TRUE(s.isMember("OutputLevel"));
+    ASSERT_TRUE(s.isMember("IsSingleResource"));
+    s.removeMember("OutputLevel");
+    s.removeMember("IsSingleResource");
+    
+    std::unique_ptr<IJob> job;
+    job.reset(unserializer.UnserializeJob(s));
+
+    ResourceModificationJob& tmp = dynamic_cast<ResourceModificationJob&>(*job);
+    ASSERT_TRUE(tmp.IsSingleResourceModification());
+    ASSERT_EQ(ResourceType_Series, tmp.GetOutputLevel());  // old, incorrect behavior
+    ASSERT_EQ(ResourceType_Series, tmp.GetModification().GetLevel());
   }
 
   {
     ResourceModificationJob job(GetContext());
     ASSERT_THROW(job.SetTranscode("nope"), OrthancException);
     job.SetTranscode(DicomTransferSyntax_JPEGProcess1);
+    job.SetSingleResourceModification(new DicomModification, ResourceType_Study, false);
 
     job.AddTrailingStep();  // Necessary since 1.7.0
     ASSERT_TRUE(CheckIdempotentSetOfInstances(unserializer, job));
@@ -844,6 +877,72 @@ TEST_F(OrthancJobsSerialization, Jobs)
     ASSERT_TRUE(tmp.IsTranscode());
     ASSERT_EQ(DicomTransferSyntax_JPEGProcess1, tmp.GetTransferSyntax());
     ASSERT_EQ(RequestOrigin_Unknown, tmp.GetOrigin().GetRequestOrigin());
+    ASSERT_TRUE(tmp.IsSingleResourceModification());
+    ASSERT_EQ(ResourceType_Study, tmp.GetOutputLevel());
+    ASSERT_EQ(ResourceType_Instance, tmp.GetModification().GetLevel());
+  }
+
+  {
+    ResourceModificationJob job(GetContext());
+    job.SetMultipleResourcesModification(new DicomModification, true);
+    job.AddInstance("toto");
+    job.AddInstance("tutu");
+    job.AddTrailingStep();  // Necessary since 1.7.0
+    ASSERT_TRUE(CheckIdempotentSetOfInstances(unserializer, job));
+    ASSERT_TRUE(job.Serialize(s));
+  }
+
+  {
+    std::unique_ptr<IJob> job;
+    job.reset(unserializer.UnserializeJob(s));
+
+    ResourceModificationJob& tmp = dynamic_cast<ResourceModificationJob&>(*job);
+
+    std::set<std::string> instances;
+    for (size_t i = 0; i < tmp.GetInstancesCount(); i++)
+    {
+      instances.insert(tmp.GetInstance(i));
+    }
+    
+    ASSERT_EQ(2u, instances.size());
+    ASSERT_TRUE(instances.find("toto") != instances.end());
+    ASSERT_TRUE(instances.find("tutu") != instances.end());
+
+    ASSERT_TRUE(tmp.IsAnonymization());
+    ASSERT_FALSE(tmp.IsSingleResourceModification());
+    ASSERT_THROW(tmp.GetOutputLevel(), OrthancException);
+    ASSERT_EQ(ResourceType_Instance, tmp.GetModification().GetLevel());
+  }
+
+  {
+    // Test behavior on broken serialization
+    ASSERT_FALSE(s.isMember("OutputLevel"));
+    ASSERT_TRUE(s.isMember("IsSingleResource"));
+    s.removeMember("IsSingleResource");
+
+    {
+      std::unique_ptr<IJob> job;
+      job.reset(unserializer.UnserializeJob(s));
+
+      ResourceModificationJob& tmp = dynamic_cast<ResourceModificationJob&>(*job);
+      ASSERT_TRUE(tmp.IsAnonymization());
+      ASSERT_TRUE(tmp.IsSingleResourceModification());
+      ASSERT_EQ(ResourceType_Patient, tmp.GetOutputLevel());
+      ASSERT_EQ(ResourceType_Instance, tmp.GetModification().GetLevel());
+    }
+
+    s["Modification"]["Level"] = "Series";
+
+    {
+      std::unique_ptr<IJob> job;
+      job.reset(unserializer.UnserializeJob(s));
+
+      ResourceModificationJob& tmp = dynamic_cast<ResourceModificationJob&>(*job);
+      ASSERT_TRUE(tmp.IsAnonymization());
+      ASSERT_TRUE(tmp.IsSingleResourceModification());
+      ASSERT_EQ(ResourceType_Series, tmp.GetOutputLevel());
+      ASSERT_EQ(ResourceType_Series, tmp.GetModification().GetLevel());
+    }
   }
 
   // SplitStudyJob
@@ -1088,6 +1187,7 @@ TEST_F(OrthancJobsSerialization, DicomAssociationParameters)
     ASSERT_EQ(104u, job->GetParameters().GetRemoteModality().GetPortNumber());
     ASSERT_EQ(ModalityManufacturer_Generic, job->GetParameters().GetRemoteModality().GetManufacturer());
     ASSERT_EQ(DicomAssociationParameters::GetDefaultTimeout(), job->GetParameters().GetTimeout());
+    ASSERT_EQ(DicomToJsonFormat_Short, job->GetQueryFormat());
   }
   
   {
@@ -1100,6 +1200,8 @@ TEST_F(OrthancJobsSerialization, DicomAssociationParameters)
     job.SetLocalAet("WORLD");
     job.SetRemoteModality(r);
     job.SetTimeout(43);
+    job.SetQueryFormat(DicomToJsonFormat_Human);
+
     job.Serialize(v);
   }
   
@@ -1113,5 +1215,63 @@ TEST_F(OrthancJobsSerialization, DicomAssociationParameters)
     ASSERT_EQ(42u, job->GetParameters().GetRemoteModality().GetPortNumber());
     ASSERT_EQ(ModalityManufacturer_Generic, job->GetParameters().GetRemoteModality().GetManufacturer());
     ASSERT_EQ(43u, job->GetParameters().GetTimeout());
+    ASSERT_EQ(DicomToJsonFormat_Human, job->GetQueryFormat());
+  }
+}
+
+
+TEST_F(OrthancJobsSerialization, DicomMoveScuJob)
+{
+  Json::Value command = Json::objectValue;
+  command["0008,0005"]["Type"] = "String";
+  command["0008,0005"]["Content"] = "ISO_IR 100";
+  command["0010,0020"]["Type"] = "String";
+  command["0010,0020"]["Content"] = "1234";
+
+  Json::Value query = Json::objectValue;
+  query["0010,0020"] = "456";
+  query["0008,0052"] = "STUDY";
+  
+  Json::Value remote = Json::objectValue;
+  remote["AET"] = "REMOTE";
+  remote["Host"] = "192.168.1.1";
+  remote["Port"] = 4242;
+  
+  Json::Value s = Json::objectValue;
+  s["Permissive"] = true;
+  s["Position"] = 1;
+  s["Description"] = "test";
+  s["Remote"] = remote;
+  s["LocalAet"] = "LOCAL";
+  s["TargetAet"] = "TARGET";
+  s["QueryFormat"] = "Full";
+  s["Query"] = Json::arrayValue;
+  s["Query"].append(query);
+  s["Commands"] = Json::arrayValue;
+  s["Commands"].append(command);
+
+  Json::Value s2;
+
+  {
+    DicomMoveScuJob job(GetContext(), s);
+    job.Serialize(s2);
+  }
+
+  {
+    DicomMoveScuJob job(GetContext(), s2);
+    ASSERT_EQ("TARGET", job.GetTargetAet());
+    ASSERT_EQ("LOCAL", job.GetParameters().GetLocalApplicationEntityTitle());
+    ASSERT_EQ("REMOTE", job.GetParameters().GetRemoteModality().GetApplicationEntityTitle());
+    ASSERT_EQ("192.168.1.1", job.GetParameters().GetRemoteModality().GetHost());
+    ASSERT_EQ(4242u, job.GetParameters().GetRemoteModality().GetPortNumber());
+    ASSERT_EQ("test", job.GetDescription());
+    ASSERT_TRUE(job.IsPermissive());
+    ASSERT_EQ(1u, job.GetPosition());
+    ASSERT_EQ(1u, job.GetCommandsCount());
+    ASSERT_EQ(DicomToJsonFormat_Full, job.GetQueryFormat());
+    ASSERT_EQ(1u, s2["Commands"].size());
+    ASSERT_EQ(command.toStyledString(), s2["Commands"][0].toStyledString());
+    ASSERT_EQ(1u, s2["Query"].size());
+    ASSERT_EQ(query.toStyledString(), s2["Query"][0].toStyledString());
   }
 }
